@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016-2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
@@ -30,26 +30,13 @@ This module contains configuration classes for Nicotine.
 
 import configparser
 import os
-import pickle
 import sys
-import time
 
 from ast import literal_eval
-from gettext import gettext as _
-from os.path import exists
+from collections import defaultdict
 
 from pynicotine.logfacility import log
-
-
-class RestrictedUnpickler(pickle.Unpickler):
-    """
-    Don't allow code execution from pickles
-    """
-
-    def find_class(self, module, name):
-        # Forbid all globals
-        raise pickle.UnpicklingError("global '%s.%s' is forbidden" %
-                                     (module, name))
+from pynicotine.utils import RestrictedUnpickler
 
 
 class Config:
@@ -59,9 +46,7 @@ class Config:
 
     need_config() - returns true if configuration information is incomplete
     read_config() - reads configuration information from file
-    setConfig(config_info_dict) - sets configuration information
     write_configuration - writes configuration information to file
-    write_download_queue - writes download queue to file
 
     The actual configuration information is stored as a two-level dictionary.
     First-level keys are config sections, second-level keys are config
@@ -72,27 +57,32 @@ class Config:
 
         self.filename = filename
         self.data_dir = data_dir
-        self.parser = configparser.RawConfigParser()
+        self.parser = configparser.RawConfigParser(strict=False)
+
+        self.create_config_folder()
+        self.create_data_folder()
 
         try:
-            self.parser.read([self.filename], encoding="utf-8")
+            self.parse_config()
+
         except UnicodeDecodeError:
             self.convert_config()
-            self.parser.read([self.filename], encoding="utf-8")
+            self.parse_config()
 
         log_dir = os.path.join(data_dir, "logs")
 
-        self.sections = {
+        self.defaults = {
             "server": {
                 "server": ('server.slsknet.org', 2242),
                 "login": '',
                 "passw": '',
-                "firewalled": False,
                 "ctcpmsgs": False,
                 "autosearch": [],
                 "autoreply": "",
                 "portrange": (2234, 2239),
                 "upnp": True,
+                "upnp_interval": 4,
+                "auto_connect_startup": True,
                 "userlist": [],
                 "banlist": [],
                 "ignorelist": [],
@@ -100,7 +90,8 @@ class Config:
                 "ipblocklist": {},
                 "autojoin": ["nicotine"],
                 "autoaway": 15,
-                "private_chatrooms": False
+                "private_chatrooms": False,
+                "command_aliases": {}
             },
 
             "transfers": {
@@ -121,19 +112,18 @@ class Config:
                 "afterfolder": "",
                 "lock": True,
                 "reverseorder": False,
-                "prioritize": False,
                 "fifoqueue": False,
                 "usecustomban": False,
                 "limitby": True,
                 "customban": "Banned, don't bother retrying",
                 "queuelimit": 10000,
-                "filelimit": 1000,
+                "filelimit": 100,
                 "friendsonly": False,
                 "friendsnolimits": False,
                 "enablebuddyshares": False,
                 "enabletransferbuttons": True,
-                "groupdownloads": True,
-                "groupuploads": True,
+                "groupdownloads": 1,
+                "groupuploads": 1,
                 "geoblock": False,
                 "geopanic": False,
                 "geoblockcc": [""],
@@ -141,19 +131,8 @@ class Config:
                 "uploadallowed": 2,
                 "autoclear_downloads": False,
                 "autoclear_uploads": False,
-                "downloads": [],
-                "sharedfiles": {},
-                "sharedfilesstreams": {},
                 "uploadsinsubdirs": True,
-                "wordindex": {},
-                "fileindex": {},
-                "sharedmtimes": {},
-                "bsharedfiles": {},
-                "bsharedfilesstreams": {},
-                "bwordindex": {},
-                "bfileindex": {},
-                "bsharedmtimes": {},
-                "rescanonstartup": 0,
+                "rescanonstartup": True,
                 "enablefilters": True,
                 "downloadregexp": "",
                 "downloadfilters": [
@@ -204,14 +183,14 @@ class Config:
                 "debug": False,
                 "debugmodes": [0, 1],
                 "debuglogsdir": os.path.join(log_dir, "debug"),
-                "logcollapsed": False,
+                "logcollapsed": True,
                 "transferslogsdir": os.path.join(log_dir, "transfers"),
                 "rooms_timestamp": "%H:%M:%S",
                 "private_timestamp": "%Y-%m-%d %H:%M:%S",
                 "log_timestamp": "%Y-%m-%d %H:%M:%S",
                 "timestamps": True,
-                "privatechat": False,
-                "chatrooms": False,
+                "privatechat": True,
+                "chatrooms": True,
                 "transfers": False,
                 "debug_file_output": False,
                 "roomlogsdir": os.path.join(log_dir, "rooms"),
@@ -223,23 +202,17 @@ class Config:
             },
 
             "privatechat": {
-                "store": False,
+                "store": True,
                 "users": []
             },
 
             "columns": {
-                "userbrowse": [1, 1, 1, 1],
-                "userbrowse_widths": [600, 100, 70, 0],
-                "userlist": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "userlist_widths": [0, 25, 180, 0, 0, 0, 0, 0, 160, 0],
-                "chatrooms": {},
-                "chatrooms_widths": {},
-                "download_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "download_widths": [200, 250, 250, 140, 50, 70, 170, 90, 140, 0],
-                "upload_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "upload_widths": [200, 250, 250, 140, 50, 70, 170, 90, 140, 0],
-                "filesearch_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "filesearch_widths": [50, 200, 25, 50, 90, 90, 400, 400, 100, 100, 0],
+                "file_search": {},
+                "download": {},
+                "upload": {},
+                "user_browse": {},
+                "buddy_list": {},
+                "chat_room": {},
                 "hideflags": False
             },
 
@@ -250,15 +223,14 @@ class Config:
                 "re_filter": False,
                 "history": [],
                 "enablefilters": False,
+                "filters_visible": False,
                 "defilter": ["", "", "", "", 0, ""],
                 "filtercc": [],
-                "reopen_tabs": False,
                 "filterin": [],
                 "filterout": [],
                 "filtersize": [],
                 "filterbr": [],
-                "distrib_timer": False,
-                "distrib_ignore": 60,
+                "filtertype": [],
                 "search_results": True,
                 "max_displayed_results": 1000,
                 "max_stored_results": 1500,
@@ -268,15 +240,16 @@ class Config:
 
             "ui": {
                 "dark_mode": False,
+                "header_bar": True,
                 "icontheme": "",
-                "chatme": "FOREST GREEN",
+                "chatme": "#908e8b",
                 "chatremote": "",
-                "chatlocal": "BLUE",
-                "chathilite": "red",
-                "urlcolor": "#3D2B7F",
-                "useronline": "BLACK",
-                "useraway": "ORANGE",
-                "useroffline": "#aa0000",
+                "chatlocal": "",
+                "chathilite": "#5288ce",
+                "urlcolor": "#5288ce",
+                "useronline": "#16bb5c",
+                "useraway": "#c9ae13",
+                "useroffline": "#e04f5e",
                 "usernamehotspots": True,
                 "usernamestyle": "bold",
                 "textbg": "",
@@ -287,8 +260,8 @@ class Config:
                 "exitdialog": 1,
                 "notexists": True,
                 "tab_default": "",
-                "tab_hilite": "red",
-                "tab_changed": "#0000ff",
+                "tab_hilite": "#497ec2",
+                "tab_changed": "#497ec2",
                 "tab_select_previous": True,
                 "tab_reorderable": True,
                 "tabmain": "Top",
@@ -306,6 +279,7 @@ class Config:
                 "labelbrowse": 0,
                 "labelsearch": 0,
                 "decimalsep": ",",
+                "globalfont": "",
                 "chatfont": "",
                 "roomlistcollapsed": False,
                 "tabclosers": True,
@@ -315,30 +289,31 @@ class Config:
                 "transfersfont": "",
                 "last_tab_id": 0,
                 "modes_visible": {
-                    "chatrooms": 1,
-                    "private": 1,
-                    "downloads": 1,
-                    "uploads": 1,
-                    "search": 1,
-                    "userinfo": 1,
-                    "userbrowse": 1,
-                    "interests": 1
+                    "search": True,
+                    "downloads": True,
+                    "uploads": True,
+                    "userbrowse": True,
+                    "userinfo": True,
+                    "private": True,
+                    "chatrooms": True,
+                    "interests": True
                 },
                 "modes_order": [
-                    "chatrooms",
-                    "private",
+                    "search",
                     "downloads",
                     "uploads",
-                    "search",
-                    "userinfo",
                     "userbrowse",
-                    "interests",
-                    "userlist"
+                    "userinfo",
+                    "private",
+                    "userlist",
+                    "chatrooms",
+                    "interests"
                 ],
                 "showaway": False,
-                "buddylistinchatrooms": 0,
+                "buddylistinchatrooms": "tab",
                 "trayicon": True,
-                "filemanager": "xdg-open $",
+                "startup_hidden": False,
+                "filemanager": "",
                 "speechenabled": False,
                 "speechprivate": "%(user)s told you.. %(message)s",
                 "speechrooms": "In %(room)s, %(user)s said %(message)s",
@@ -367,9 +342,9 @@ class Config:
             },
 
             "players": {
-                "default": "xdg-open $",
+                "default": "",
                 "npothercommand": "",
-                "npplayer": "",
+                "npplayer": "mpris",
                 "npformatlist": [],
                 "npformat": ""
             },
@@ -378,7 +353,7 @@ class Config:
                 "notification_window_title": True,
                 "notification_tab_colors": False,
                 "notification_tab_icons": True,
-                "notification_popup_sound": True,
+                "notification_popup_sound": False,
                 "notification_popup_file": True,
                 "notification_popup_folder": True,
                 "notification_popup_private_message": True,
@@ -389,30 +364,84 @@ class Config:
             "plugins": {
                 "enable": True,
                 "enabled": []
+            },
+
+            "statistics": {
+                "started_downloads": 0,
+                "completed_downloads": 0,
+                "downloaded_size": 0,
+                "started_uploads": 0,
+                "completed_uploads": 0,
+                "uploaded_size": 0
             }
         }
 
         # Windows specific stuff
-        if sys.platform.startswith('win'):
-            self.sections['ui']['filemanager'] = 'explorer $'
+        if sys.platform == "win32":
+            self.defaults['players']['npplayer'] = 'other'
 
-        self.defaults = {}
-        for key, value in self.sections.items():
-            if isinstance(value, dict):
-                if key not in self.defaults:
-                    self.defaults[key] = {}
+        # Clean up old config options
+        self.remove_old_options()
 
-                for key2, value2 in value.items():
-                    self.defaults[key][key2] = value2
-            else:
-                self.defaults[key] = value
+        # Initialize config with default values
+        self.sections = defaultdict(dict)
+
+        for key, value in self.defaults.items():
+            self.sections[key] = value.copy()
+
+        # Update config values from file
+        self.set_config()
+
+        # Load command aliases from legacy file
+        try:
+            if not self.sections["server"]["command_aliases"] and os.path.exists(filename + ".alias"):
+                with open(filename + ".alias", 'rb') as f:
+                    self.sections["server"]["command_aliases"] = RestrictedUnpickler(f, encoding='utf-8').load()
+
+        except Exception:
+            return
+
+    def create_config_folder(self):
+        """ Create the folder for storing the config file in, if the folder
+        doesn't exist """
+
+        path, fn = os.path.split(self.filename)
+        try:
+            if not os.path.isdir(path):
+                os.makedirs(path)
+
+        except OSError as msg:
+            log.add_warning(_("Can't create directory '%(path)s', reported error: %(error)s"), {'path': path, 'error': msg})
+            return False
+
+        return True
+
+    def create_data_folder(self):
+        """ Create the folder for storing data in (aliases, shared files etc.),
+        if the folder doesn't exist """
 
         try:
-            f = open(filename + ".alias", 'rb')
-            self.aliases = RestrictedUnpickler(f).load()
-            f.close()
-        except Exception:
-            self.aliases = {}
+            if not os.path.isdir(self.data_dir):
+                os.makedirs(self.data_dir)
+
+        except OSError as msg:
+            log.add_warning(_("Can't create directory '%(path)s', reported error: %(error)s"), {'path': self.data_dir, 'error': msg})
+
+    def parse_config(self):
+        """ Parses the config file """
+
+        try:
+            with open(self.filename, 'a+', encoding="utf-8") as f:
+                f.seek(0)
+                self.parser.read_file(f)
+
+        except configparser.ParsingError:
+            # Ignore parsing errors, the offending lines are removed later
+            pass
+
+        except Exception as e:
+            # Miscellaneous failure, default config will be used
+            log.add(_("Unable to parse config file: %s"), e)
 
     def convert_config(self):
         """ Converts the config to utf-8.
@@ -420,6 +449,7 @@ class Config:
 
         try:
             from chardet import detect
+
         except ImportError:
             print("Failed to convert config file to UTF-8. Please install python3-chardet and start Nicotine+ again.")
             sys.exit()
@@ -440,78 +470,96 @@ class Config:
 
     def need_config(self):
 
-        errorlevel = 0
+        # Check if we have specified a username or password
+        if not self.sections["server"]["login"] or \
+                not self.sections["server"]["passw"]:
 
-        try:
-            for i in self.sections:
-                for j in self.sections[i]:
+            log.add(_("You need to specify a username and password before connecting..."))
+            return True
 
-                    if not isinstance(self.sections[i][j], (type(None), type(""))):
+        return False
+
+    def set_config(self):
+        """ Set config values parsed from file earlier """
+
+        for i in self.parser.sections():
+            for j, val in self.parser.items(i, raw=True):
+
+                # Check if config section exists in defaults
+                if i not in self.defaults:
+                    log.add_warning(_("Unknown config section '%s'"), i)
+
+                # Check if config option exists in defaults
+                elif j not in self.defaults[i] and i != "plugins" and j != "filter":
+                    log.add_warning(_("Unknown config option '%(option)s' in section '%(section)s'"), {'option': j, 'section': i})
+
+                else:
+                    """ Attempt to get the default value for a config option. If there's no default
+                    value, it's a custom option from a plugin, so no checks are needed. """
+
+                    try:
+                        default_val = self.defaults[i][j]
+
+                    except KeyError:
+                        try:
+                            val = literal_eval(val)
+                        except Exception:
+                            pass
+
+                        self.sections[i][j] = val
                         continue
 
-                    if self.sections[i][j] is None or self.sections[i][j] == '' \
-                       and i not in ("userinfo", "ui", "players") \
-                       and j not in ("incompletedir", "autoreply", 'afterfinish', 'afterfolder', 'geoblockcc', 'downloadregexp'):
+                    """ Check that the value of a config option is of the same type as the default
+                    value. If that's not the case, reset the value. """
 
-                        # Repair options set to None with defaults
-                        if self.sections[i][j] is None and self.defaults[i][j] is not None:
+                    try:
+                        if not isinstance(default_val, str):
+                            # Values are always read as strings, evaluate them if they aren't
+                            # supposed to remain as strings
+                            eval_val = literal_eval(val)
 
-                            self.sections[i][j] = self.defaults[i][j]
-                            log.add(
-                                _("Config option reset to default: Section: %(section)s, Option: %(option)s, to: %(default)s"), {
-                                    'section': i,
-                                    'option': j,
-                                    'default': self.sections[i][j]
-                                }
-                            )
-
-                            if errorlevel == 0:
-                                errorlevel = 1
                         else:
+                            eval_val = val
 
-                            if errorlevel < 2:
-                                log.add(_("You need to configure your settings (Server, Username, Password, Download Directory) before connecting..."))
-                                errorlevel = 2
+                        if i != "plugins" and j != "filter":
+                            if (isinstance(default_val, bool) and isinstance(eval_val, int) and eval_val != 0 and eval_val != 1) or \
+                                    (not isinstance(default_val, bool) and type(eval_val) != type(default_val)):
 
-        except Exception as error:
-            log.add(_("Config error: %s"), error)
-            if errorlevel < 3:
-                errorlevel = 3
+                                raise TypeError("Invalid config value type detected")
 
-        return errorlevel
+                        self.sections[i][j] = eval_val
 
-    def read_config(self):
+                    except Exception:
+                        # Value was unexpected, reset option
+                        self.sections[i][j] = default_val
 
-        self.sections['transfers']['downloads'] = []
+                        log.add_warning("Config error: Couldn't decode '%s' section '%s' value '%s', value has been reset", (
+                            (i[:120] + '..') if len(i) > 120 else i,
+                            (j[:120] + '..') if len(j) > 120 else j,
+                            (val[:120] + '..') if len(val) > 120 else val
+                        )
+                        )
 
-        if exists(os.path.join(self.data_dir, 'transfers.pickle')):
-            # <1.2.13 stored transfers inside the main config
-            try:
-                handle = open(os.path.join(self.data_dir, 'transfers.pickle'), 'rb')
-            except IOError as inst:
-                log.add_warning(_("Something went wrong while opening your transfer list: %(error)s"), {'error': str(inst)})
-            else:
-                try:
-                    self.sections['transfers']['downloads'] = RestrictedUnpickler(handle).load()
-                except Exception as inst:
-                    log.add_warning(_("Something went wrong while reading your transfer list: %(error)s"), {'error': str(inst)})
-            try:
-                handle.close()
-            except Exception:
-                pass
+        server = self.sections["server"]
 
-        path, fn = os.path.split(self.filename)
-        try:
-            if not os.path.isdir(path):
-                os.makedirs(path)
-        except OSError as msg:
-            log.add_warning("Can't create directory '%s', reported error: %s", (path, msg))
+        # Check if server value is valid
+        if len(server["server"]) != 2 or \
+                not isinstance(server["server"][0], str) or \
+                not isinstance(server["server"][1], int):
 
-        try:
-            if not os.path.isdir(self.data_dir):
-                os.makedirs(self.data_dir)
-        except OSError as msg:
-            log.add_warning("Can't create directory '%s', reported error: %s", (path, msg))
+            server["server"] = self.defaults["server"]["server"]
+
+        # Check if port range value is valid
+        if len(server["portrange"]) != 2 or \
+                not all(isinstance(i, int) for i in server["portrange"]):
+
+            server["portrange"] = self.defaults["server"]["portrange"]
+
+        else:
+            # Setting the port range in numerical order
+            server["portrange"] = (min(server["portrange"]), max(server["portrange"]))
+
+    def remove_old_options(self):
 
         # Transition from 1.2.16 -> 1.4.0
         # Do the cleanup early so we don't get the annoying
@@ -575,236 +623,97 @@ class Config:
         # Remove old log folder option, superseded by individual log folders for transfers and debug messages
         self.remove_old_option("logging", "logsdir")
 
-        # Checking for unknown section/options
-        unknown1 = [
-            'login', 'passw', 'enc', 'downloaddir', 'uploaddir', 'customban',
-            'descr', 'pic', 'transferslogsdir', 'roomlogsdir', 'privatelogsdir',
-            'incompletedir', 'autoreply', 'afterfinish', 'downloadregexp',
-            'afterfolder', 'default', 'chatfont', 'npothercommand', 'npplayer',
-            'npformat', 'private_timestamp', 'rooms_timestamp', 'log_timestamp',
-            'debuglogsdir'
-        ]
+        # Remove option to stop responding to searches for a certain time
+        self.remove_old_option("searches", "distrib_timer")
+        self.remove_old_option("searches", "distrib_ignore")
 
-        unknown2 = {
-            'ui': [
-                "roomlistcollapsed", "tab_select_previous", "tabclosers",
-                "tab_colors", "tab_reorderable", "buddylistinchatrooms", "trayicon",
-                "showaway", "usernamehotspots", "exitdialog",
-                "tab_icons", "spellcheck", "modes_order", "modes_visible",
-                "chat_hidebuttons", "tab_status_icons", "notexists",
-                "speechenabled", "enablefilters", "width",
-                "height", "xposition", "yposition", "labelmain", "labelrooms",
-                "labelprivate", "labelinfo", "labelbrowse", "labelsearch", "maximized",
-                "dark_mode"
-            ],
-            'words': [
-                "completion", "censorwords", "replacewords", "autoreplaced",
-                "censored", "characters", "tab", "cycle", "dropdown",
-                "roomnames", "buddies", "roomusers", "commands",
-                "aliases", "onematch"
-            ]
-        }
+        # Remove "I can receive direct connections"-option, it's redundant now
+        self.remove_old_option("server", "firewalled")
 
-        for i in self.parser.sections():
-            for j in self.parser.options(i):
-                val = self.parser.get(i, j, raw=1)
-                if i not in self.sections:
-                    log.add_warning(_("Unknown config section '%s'"), i)
-                elif j not in self.sections[i] and not (j == "filter" or i in ('plugins',)):
-                    log.add_warning(_("Unknown config option '%(option)s' in section '%(section)s'"), {'option': j, 'section': i})
-                elif j in unknown1 or (i in unknown2 and j not in unknown2[i]):
-                    if val is not None and val != "None":
-                        self.sections[i][j] = val
-                    else:
-                        self.sections[i][j] = None
-                else:
-                    try:
-                        self.sections[i][j] = literal_eval(val)
-                    except Exception:
-                        self.sections[i][j] = None
-                        log.add_warning("CONFIG ERROR: Couldn't decode '%s' section '%s' value '%s'", (str(j), str(i), str(val)))
+        # Remove old column options
+        self.remove_old_option("columns", "userbrowse")
+        self.remove_old_option("columns", "userbrowse_widths")
+        self.remove_old_option("columns", "userlist")
+        self.remove_old_option("columns", "userlist_widths")
+        self.remove_old_option("columns", "chatrooms")
+        self.remove_old_option("columns", "chatrooms_widths")
+        self.remove_old_option("columns", "download_columns")
+        self.remove_old_option("columns", "download_widths")
+        self.remove_old_option("columns", "upload_columns")
+        self.remove_old_option("columns", "upload_widths")
+        self.remove_old_option("columns", "filesearch_columns")
+        self.remove_old_option("columns", "filesearch_widths")
 
-        # Setting the port range in numerical order
-        self.sections["server"]["portrange"] = (min(self.sections["server"]["portrange"]), max(self.sections["server"]["portrange"]))
+        # Remove option to prioritize sfv/md5 files when downloading
+        self.remove_old_option("transfers", "prioritize")
+
+        # Remove option to reopen closed search tabs when new results come in
+        self.remove_old_option("searches", "reopen_tabs")
 
     def remove_old_option(self, section, option):
+
         if section in self.parser.sections() and option in self.parser.options(section):
             self.parser.remove_option(section, option)
 
     def remove_old_section(self, section):
+
         if section in self.parser.sections():
             self.parser.remove_section(section)
 
-    def write_download_queue(self):
-
-        realfile = os.path.join(self.data_dir, 'transfers.pickle')
-        tmpfile = realfile + '.tmp'
-        backupfile = realfile + ' .backup'
-        try:
-            handle = open(tmpfile, 'wb')
-        except Exception as inst:
-            log.add_warning(_("Something went wrong while opening your transfer list: %(error)s"), {'error': str(inst)})
-        else:
-            try:
-                pickle.dump(self.sections['transfers']['downloads'], handle, protocol=pickle.HIGHEST_PROTOCOL)
-                handle.close()
-                try:
-                    # Please let it be atomic...
-                    os.rename(tmpfile, realfile)
-                except Exception:
-                    # ...ugh. Okay, how about...
-                    try:
-                        os.unlink(backupfile)
-                    except Exception:
-                        pass
-                    os.rename(realfile, backupfile)
-                    os.rename(tmpfile, realfile)
-            except Exception as inst:
-                log.add_warning(_("Something went wrong while writing your transfer list: %(error)s"), {'error': str(inst)})
-        finally:
-            try:
-                handle.close()
-            except Exception:
-                pass
-
     def write_configuration(self):
 
-        external_sections = [
-            "sharedfiles", "sharedfilesstreams", "wordindex", "fileindex",
-            "sharedmtimes", "bsharedfiles", "bsharedfilesstreams",
-            "bwordindex", "bfileindex", "bsharedmtimes", "downloads"
-        ]
-
         for i in self.sections:
             if not self.parser.has_section(i):
                 self.parser.add_section(i)
+
             for j in self.sections[i]:
-                if j not in external_sections:
-                    self.parser.set(i, j, self.sections[i][j])
-                else:
-                    self.parser.remove_option(i, j)
+                self.parser.set(i, j, self.sections[i][j])
 
-        path, fn = os.path.split(self.filename)
+        if not self.create_config_folder():
+            return
+
+        # Back up old config to config.old
         try:
-            if not os.path.isdir(path):
-                os.makedirs(path)
-        except OSError as msg:
-            log.add_warning(_("Can't create directory '%(path)s', reported error: %(error)s"), {'path': path, 'error': msg})
+            if os.path.exists(self.filename):
+                from shutil import copy2
+                copy2(self.filename, self.filename + ".old")
 
+                # A paranoid precaution since config contains the password
+                os.chmod(self.filename + ".old", 0o600)
+
+        except Exception as error:
+            log.add(_("Unable to back up config file: %s"), error)
+
+        # Save new config to file
         oldumask = os.umask(0o077)
 
         try:
-            f = open(self.filename + ".new", "w", encoding="utf-8")
-        except IOError as e:
-            log.add_warning(_("Can't save config file, I/O error: %s"), e)
-            return
-        else:
-            try:
+            with open(self.filename, "w", encoding="utf-8") as f:
                 self.parser.write(f)
-            except IOError as e:
-                log.add_warning(_("Can't save config file, I/O error: %s"), e)
-                return
-            else:
-                f.close()
+
+        except Exception as e:
+            log.add(_("Unable to save config file: %s"), e)
 
         os.umask(oldumask)
 
-        # A paranoid precaution since config contains the password
-        try:
-            os.chmod(self.filename, 0o600)
-        except Exception:
-            pass
+    def write_config_backup(self, filename):
 
-        try:
-            s = os.stat(self.filename)
-            if s.st_size > 0:
-                try:
-                    if os.path.exists(self.filename + ".old"):
-                        os.remove(self.filename + ".old")
-                except OSError:
-                    log.add_warning(_("Can't remove %s", self.filename + ".old"))
-                try:
-                    os.rename(self.filename, self.filename + ".old")
-                except OSError as error:
-                    log.add_warning(_("Can't back config file up, error: %s"), error)
-        except OSError:
-            pass
+        if not filename.endswith(".tar.bz2"):
+            filename += ".tar.bz2"
 
-        try:
-            os.rename(self.filename + ".new", self.filename)
-        except OSError as error:
-            log.add_warning(_("Can't rename config file, error: %s"), error)
-
-    def write_config_backup(self, filename=None):
-
-        if filename is None:
-            filename = "%s backup %s.tar.bz2", (self.filename, time.strftime("%Y-%m-%d %H:%M:%S"))
-        else:
-            if filename[-8:-1] != ".tar.bz2":
-                filename += ".tar.bz2"
         try:
             if os.path.exists(filename):
                 raise FileExistsError("File %s exists", filename)
 
             import tarfile
-            tar = tarfile.open(filename, "w:bz2")
+            with tarfile.open(filename, "w:bz2") as tar:
+                if not os.path.exists(self.filename):
+                    raise FileNotFoundError("Config file missing")
 
-            if not os.path.exists(self.filename):
-                raise FileNotFoundError("Config file missing")
+                tar.add(self.filename)
 
-            tar.add(self.filename)
-
-            if os.path.exists(self.filename + ".alias"):
-                tar.add(self.filename + ".alias")
-
-            tar.close()
         except Exception as e:
             print(e)
-            return (1, "Cannot write backup archive: %s", e)
+            return (True, "Cannot write backup archive: %s" % e)
 
-        return (0, filename)
-
-    def write_aliases(self):
-
-        try:
-            f = open(self.filename + ".alias", "wb")
-        except Exception as e:
-            log.add_warning(_("Something went wrong while opening your alias file: %s"), e)
-        else:
-            try:
-                pickle.dump(self.aliases, f, protocol=pickle.HIGHEST_PROTOCOL)
-                f.close()
-            except Exception as e:
-                log.add_warning(_("Something went wrong while saving your alias file: %s"), e)
-        finally:
-            try:
-                f.close()
-            except Exception:
-                pass
-
-    def add_alias(self, rest):
-        if rest:
-            args = rest.split(" ", 1)
-            if len(args) == 2:
-                if args[0] in ("alias", "unalias"):
-                    return "I will not alias that!\n"
-                self.aliases[args[0]] = args[1]
-                self.write_aliases()
-            if args[0] in self.aliases:
-                return "Alias %s: %s\n" % (args[0], self.aliases[args[0]])
-            else:
-                return _("No such alias (%s)") % rest + "\n"
-        else:
-            m = "\n" + _("Aliases:") + "\n"
-            for (key, value) in self.aliases.items():
-                m = m + "%s: %s\n" % (key, value)
-            return m + "\n"
-
-    def unalias(self, rest):
-        if rest and rest in self.aliases:
-            x = self.aliases[rest]
-            del self.aliases[rest]
-            self.write_aliases()
-            return _("Removed alias %(alias)s: %(action)s\n") % {'alias': rest, 'action': x}
-        else:
-            return _("No such alias (%(alias)s)\n") % {'alias': rest}
+        return (False, filename)
