@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2007 Gallows <g4ll0ws@gmail.com>
@@ -20,9 +20,10 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import os
+
 from collections import deque
-from gettext import gettext as _
 from time import altzone
 from time import daylight
 
@@ -35,16 +36,21 @@ from gi.repository import Pango
 
 from pynicotine import slskmessages
 from pynicotine.gtkgui.chatrooms import get_completion
+from pynicotine.gtkgui.dialogs import option_dialog
+from pynicotine.gtkgui.utils import add_alias
 from pynicotine.gtkgui.utils import append_line
 from pynicotine.gtkgui.utils import expand_alias
 from pynicotine.gtkgui.utils import IconNotebook
 from pynicotine.gtkgui.utils import is_alias
 from pynicotine.gtkgui.utils import load_ui_elements
+from pynicotine.gtkgui.utils import open_log
 from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import scroll_bottom
 from pynicotine.gtkgui.utils import TextSearchBar
 from pynicotine.gtkgui.utils import set_widget_color
 from pynicotine.gtkgui.utils import set_widget_font
+from pynicotine.gtkgui.utils import triggers_context_menu
+from pynicotine.gtkgui.utils import unalias
 from pynicotine.gtkgui.utils import update_widget_visuals
 from pynicotine.logfacility import log
 from pynicotine.utils import clean_file
@@ -88,11 +94,15 @@ class PrivateChats(IconNotebook):
         self.users = {}
         self.clist = []
 
-        self.notebook.connect("switch-page", self.on_switch_page)
+        self.notebook.connect("switch-page", self.on_switch_chat)
 
-    def on_switch_page(self, notebook, page, page_num, forceupdate=False):
+        # Clear list of previously open chats if we don't want to restore them
+        if not frame.np.config.sections["privatechat"]["store"]:
+            frame.np.config.sections["privatechat"]["users"].clear()
 
-        if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) and not forceupdate:
+    def on_switch_chat(self, notebook, page, page_num, forceupdate=False):
+
+        if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatechatvbox) and not forceupdate:
             return
 
         for user, tab in list(self.users.items()):
@@ -104,7 +114,7 @@ class PrivateChats(IconNotebook):
 
     def clear_notifications(self):
 
-        if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox):
+        if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatechatvbox):
             return
 
         page = self.get_nth_page(self.get_current_page())
@@ -146,6 +156,9 @@ class PrivateChats(IconNotebook):
             tab = PrivateChat(self, user)
             self.users[user] = tab
 
+            if user not in self.frame.np.config.sections["privatechat"]["users"]:
+                self.frame.np.config.sections["privatechat"]["users"].append(user)
+
             if not self.frame.np.config.sections["ui"]["tab_status_icons"]:
                 userlabel = "%s (%s)" % (user[:15], _("Offline"))
             else:
@@ -161,64 +174,45 @@ class PrivateChats(IconNotebook):
         if text is not None:
             self.users[user].send_message(text, bytestring=bytestring)
 
-    def tab_popup(self, user):
+    def on_tab_popup(self, widget, page):
 
-        if user not in self.users:
-            return
-
-        popup = PopupMenu(self.frame)
-        popup.setup(
-            ("#" + _("Show IP a_ddress"), popup.on_show_ip_address),
-            ("#" + _("Get user i_nfo"), popup.on_get_user_info),
-            ("#" + _("Brow_se files"), popup.on_browse_user),
-            ("#" + _("Gi_ve privileges"), popup.on_give_privileges),
-            ("#" + _("Client Version"), popup.on_version),
-            ("", None),
-            ("$" + _("Add user to list"), popup.on_add_to_list),
-            ("$" + _("Ban this user"), popup.on_ban_user),
-            ("$" + _("Ignore this user"), popup.on_ignore_user),
-            ("$" + _("B_lock this user's IP Address"), popup.on_block_user),
-            ("$" + _("Ignore this user's IP Address"), popup.on_ignore_ip),
-            ("", None),
-            ("#" + _("Close this tab"), self.users[user].on_close)
-        )
-
-        popup.set_user(user)
-
-        me = (popup.user is None or popup.user == self.frame.np.config.sections["server"]["login"])
-        items = popup.get_children()
-
-        items[6].set_active(user in (i[0] for i in self.frame.np.config.sections["server"]["userlist"]))
-        items[7].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-        items[8].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-        items[9].set_active(self.frame.user_ip_is_blocked(user))
-        items[9].set_sensitive(not me)
-        items[10].set_active(self.frame.user_ip_is_ignored(user))
-        items[10].set_sensitive(not me)
-
-        return popup
-
-    def on_tab_click(self, widget, event, child):
-
-        if event.type == Gdk.EventType.BUTTON_PRESS:
-
-            n = self.page_num(child)
-            page = self.get_nth_page(n)
-            username = next(user for user, tab in list(self.users.items()) if tab.Main is page)
-
-            if event.button == 2:
-                self.users[username].on_close(widget)
-                return True
-
-            if event.button == 3:
-                menu = self.tab_popup(username)
-                menu.popup(None, None, None, None, event.button, event.time)
-                return True
+        username = self.get_page_owner(page, self.users)
 
+        if username not in self.users:
             return False
 
+        menu = PopupMenu(self.frame)
+        menu.setup_user_menu(username)
+        menu.get_items()[_("Send _Message")].set_visible(False)
+
+        menu.append_item(("", None))
+        menu.append_item(("#" + _("Close All Tabs"), menu.on_close_all_tabs, self))
+        menu.append_item(("#" + _("_Close Tab"), self.users[username].on_close))
+        menu.toggle_user_items()
+
+        menu.popup()
+        return True
+
+    def on_tab_click(self, widget, event, page):
+
+        if triggers_context_menu(event):
+            return self.on_tab_popup(widget, page)
+
+        if event.button == 2:
+            username = self.get_page_owner(page, self.users)
+            self.users[username].on_close(widget)
+            return True
+
         return False
 
+    def close_all_tabs(self, dialog, response, data):
+
+        if response == Gtk.ResponseType.OK:
+            for user in self.users.copy():
+                self.users[user].on_close(dialog)
+
+        dialog.destroy()
+
     def show_message(self, msg, text, newmessage=True):
 
         if msg.user in self.frame.np.config.sections["server"]["ignorelist"]:
@@ -250,7 +244,7 @@ class PrivateChats(IconNotebook):
         # Show notifications if the private chats notebook isn't selected,
         # the tab is not selected, or the main window isn't mapped
         if self.get_current_page() != self.page_num(chat.Main) or \
-           self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) or \
+           self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatechatvbox) or \
            not self.frame.MainWindow.get_property("visible"):
             self.frame.notifications.add("private", msg.user)
 
@@ -270,7 +264,7 @@ class PrivateChats(IconNotebook):
         self.users[msg.user].show_message(text, newmessage, msg.timestamp)
 
         if ctcpversion and self.frame.np.config.sections["server"]["ctcpmsgs"] == 0:
-            self.send_message(msg.user, "Nicotine+ %s" % version)
+            self.send_message(msg.user, GLib.get_application_name() + " " + version)
 
         self.frame.np.pluginhandler.incoming_private_chat_notification(msg.user, text)
 
@@ -301,11 +295,13 @@ class PrivateChats(IconNotebook):
         for user in self.users:
             self.users[user].login()
 
-        if self.frame.np.config.sections["privatechat"]["store"]:
-            self.frame.np.config.sections["privatechat"]["users"].sort()
-            for user in self.frame.np.config.sections["privatechat"]["users"]:
-                if user not in self.users:
-                    self.send_message(user, show_user=True)
+        if not self.frame.np.config.sections["privatechat"]["store"]:
+            return
+
+        self.frame.np.config.sections["privatechat"]["users"].sort()
+        for user in self.frame.np.config.sections["privatechat"]["users"]:
+            if user not in self.users:
+                self.send_message(user, show_user=True)
 
     def conn_close(self):
 
@@ -332,7 +328,7 @@ class PrivateChats(IconNotebook):
             clist += [i[0] for i in self.frame.np.config.sections["server"]["userlist"]]
 
         if config["aliases"]:
-            clist += ["/" + k for k in list(self.frame.np.config.aliases.keys())]
+            clist += ["/" + k for k in list(self.frame.np.config.sections["server"]["command_aliases"].keys())]
 
         if config["commands"]:
             clist += self.CMDS
@@ -387,21 +383,8 @@ class PrivateChat:
         self.Log.set_active(self.frame.np.config.sections["logging"]["privatechat"])
 
         self.popup_menu_user = popup = PopupMenu(self.frame, False)
-        popup.setup(
-            ("#" + _("Show IP a_ddress"), popup.on_show_ip_address),
-            ("#" + _("Get user i_nfo"), popup.on_get_user_info),
-            ("#" + _("Brow_se files"), popup.on_browse_user),
-            ("#" + _("Gi_ve privileges"), popup.on_give_privileges),
-            ("#" + _("Client Version"), popup.on_version),
-            ("", None),
-            ("$" + _("Add user to list"), popup.on_add_to_list),
-            ("$" + _("Ban this user"), popup.on_ban_user),
-            ("$" + _("Ignore this user"), popup.on_ignore_user),
-            ("$" + _("B_lock this user's IP Address"), popup.on_block_user),
-            ("$" + _("Ignore this user's IP Address"), popup.on_ignore_ip)
-        )
-
-        popup.set_user(user)
+        popup.setup_user_menu(user)
+        popup.get_items()[_("Send _Message")].set_visible(False)
 
         self.popup_menu = popup = PopupMenu(self.frame)
         popup.setup(
@@ -412,16 +395,15 @@ class PrivateChat:
             ("#" + _("Copy"), self.on_copy_chat_log),
             ("#" + _("Copy All"), self.on_copy_all_chat_log),
             ("", None),
-            ("#" + _("Clear log"), self.on_clear_chat_log),
+            ("#" + _("View Chat Log"), self.on_view_chat_log),
+            ("#" + _("Delete Chat Log"), self.on_delete_chat_log),
             ("", None),
-            ("#" + _("Close"), self.on_close)
+            ("#" + _("Clear Message View"), self.on_clear_messages),
+            ("#" + _("_Close Tab"), self.on_close)
         )
 
         popup.set_user(user)
 
-        self.ChatScroll.connect("button_press_event", self.on_popup_menu)
-        self.ChatScroll.connect("key_press_event", self.on_popup_menu)
-
         self.create_tags()
         self.update_visuals()
 
@@ -441,17 +423,26 @@ class PrivateChat:
             numlines = 15
 
         try:
-            with open(log, 'r', encoding='utf-8') as lines:
-                # Only show as many log lines as specified in config
-                lines = deque(lines, numlines)
+            try:
+                self.append_log_lines(log, numlines, 'utf-8')
+
+            except UnicodeDecodeError:
+                self.append_log_lines(log, numlines, 'latin-1')
 
-                for line in lines:
-                    append_line(self.ChatScroll, line, self.tag_hilite, timestamp_format="", username=self.user, usertag=self.tag_hilite)
         except IOError:
             pass
 
         GLib.idle_add(scroll_bottom, self.ChatScroll.get_parent())
 
+    def append_log_lines(self, log, numlines, encoding='utf-8'):
+
+        with open(log, 'r', encoding=encoding) as lines:
+            # Only show as many log lines as specified in config
+            lines = deque(lines, numlines)
+
+            for line in lines:
+                append_line(self.ChatScroll, line, self.tag_hilite, timestamp_format="", username=self.user, usertag=self.tag_hilite)
+
     def login(self):
         timestamp_format = self.frame.np.config.sections["logging"]["private_timestamp"]
         append_line(self.ChatScroll, _("--- reconnected ---"), self.tag_hilite, timestamp_format=timestamp_format)
@@ -464,37 +455,19 @@ class PrivateChat:
         self.offlinemessage = 0
         self.update_tags()
 
-    def on_popup_menu(self, widget, event):
+    def on_message_view_clicked(self, widget, event):
 
-        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 3:
-
-            self.popup_menu.popup(None, None, None, None, event.button, event.time)
-            self.ChatScroll.stop_emission_by_name("button_press_event")
-            return True
-
-        elif event.type == Gdk.EventType.KEY_PRESS:
-
-            if event.keyval == Gdk.keyval_from_name("Menu"):
-
-                self.popup_menu.popup(None, None, None, None, 0, 0)
-                self.ChatScroll.stop_emission_by_name("key_press_event")
-                return True
+        if triggers_context_menu(event):
+            return self.on_popup_menu(widget)
 
         return False
 
+    def on_popup_menu(self, widget):
+        self.popup_menu.popup()
+        return True
+
     def on_popup_menu_user(self, widget):
-
-        items = self.popup_menu_user.get_children()
-        me = (self.popup_menu_user.user is None or self.popup_menu_user.user == self.frame.np.config.sections["server"]["login"])
-
-        items[6].set_active(self.user in (i[0] for i in self.frame.np.config.sections["server"]["userlist"]))
-        items[7].set_active(self.user in self.frame.np.config.sections["server"]["banlist"])
-        items[8].set_active(self.user in self.frame.np.config.sections["server"]["ignorelist"])
-        items[9].set_active(self.frame.user_ip_is_blocked(self.user))
-        items[9].set_sensitive(not me)
-        items[10].set_active(self.frame.user_ip_is_ignored(self.user))
-        items[10].set_sensitive(not me)
-
+        self.popup_menu_user.toggle_user_items()
         return True
 
     def on_find_chat_log(self, widget):
@@ -513,11 +486,56 @@ class PrivateChat:
         log = self.ChatScroll.get_buffer().get_text(start, end, True)
         self.frame.clip.set_text(log, -1)
 
-    def on_clear_chat_log(self, widget):
+    def on_view_chat_log(self, widget):
+        open_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user)
+
+    def delete_chat_log_response(self, dialog, response, data):
+
+        if response == Gtk.ResponseType.OK:
+            log_path = os.path.join(
+                self.frame.np.config.sections["logging"]["privatelogsdir"],
+                clean_file(self.user.replace(os.sep, "-")) + ".log"
+            )
+
+            try:
+                if os.path.exists(log_path):
+                    os.remove(log_path)
+
+            except Exception as e:
+                log.add(_("Failed to remove logged chat messages for user '%(user)s'. Error: %(error)s"), {
+                    "user": self.user,
+                    "error": e
+                })
+
+            self.on_clear_messages(dialog)
+
+        dialog.destroy()
+
+    def on_delete_chat_log(self, widget):
+
+        option_dialog(
+            parent=self.frame.MainWindow,
+            title=_('Delete Logged Messages?'),
+            message=_('Are you sure you wish to permanently delete all logged messages for this user?'),
+            callback=self.delete_chat_log_response
+        )
+
+    def on_clear_messages(self, widget):
         self.ChatScroll.get_buffer().set_text("")
 
     def on_show_chat_help(self, widget):
-        self.frame.on_about_private_chat_commands(widget)
+
+        if not hasattr(self, "AboutPrivateChatCommandsPopover"):
+            load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "popovers", "privatechatcommands.ui"))
+            self.AboutPrivateChatCommandsPopover.set_relative_to(self.ShowChatHelp)
+
+        try:
+            self.AboutPrivateChatCommandsPopover.popup()
+
+        except AttributeError:
+            # GTK <3.22 support
+            self.AboutPrivateChatCommandsPopover.set_transitions_enabled(True)
+            self.AboutPrivateChatCommandsPopover.show()
 
     def show_message(self, text, newmessage=True, timestamp=None):
 
@@ -618,7 +636,7 @@ class PrivateChat:
 
     def thread_alias(self, alias):
 
-        text = expand_alias(self.frame.np.config.aliases, alias)
+        text = expand_alias(alias)
         if not text:
             log.add(_('Alias "%s" returned nothing'), alias)
             return
@@ -636,7 +654,7 @@ class PrivateChat:
             widget.set_text("")
             return
 
-        if is_alias(self.frame.np.config.aliases, text):
+        if is_alias(text):
             import _thread
             _thread.start_new_thread(self.thread_alias, (text,))
             widget.set_text("")
@@ -652,16 +670,16 @@ class PrivateChat:
 
         if cmd in ("/alias", "/al"):
 
-            append_line(self.ChatScroll, self.frame.np.config.add_alias(realargs), None, "")
+            append_line(self.ChatScroll, add_alias(realargs), None, "")
             if self.frame.np.config.sections["words"]["aliases"]:
-                self.frame.chatrooms.roomsctrl.update_completions()
+                self.frame.chatrooms.update_completions()
                 self.frame.privatechats.update_completions()
 
         elif cmd in ("/unalias", "/un"):
 
-            append_line(self.ChatScroll, self.frame.np.config.unalias(realargs), None, "")
+            append_line(self.ChatScroll, unalias(realargs), None, "")
             if self.frame.np.config.sections["words"]["aliases"]:
-                self.frame.chatrooms.roomsctrl.update_completions()
+                self.frame.chatrooms.update_completions()
                 self.frame.privatechats.update_completions()
 
         elif cmd in ["/join", "/j"]:
@@ -790,7 +808,7 @@ class PrivateChat:
         widget.set_text("")
 
     def display_now_playing(self):
-        self.frame.now_playing.display_now_playing(callback=self.send_message)
+        self.frame.np.now_playing.display_now_playing(callback=self.send_message)
 
     def update_visuals(self):
 
