@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
@@ -22,29 +22,43 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gettext import gettext as _
+import os
+import _thread
+
 from sys import maxsize
 from time import time
 
+from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import Gtk
 
-from pynicotine.gtkgui.utils import hide_columns
+from pynicotine.gtkgui.utils import collapse_treeview
 from pynicotine.gtkgui.utils import human_size
 from pynicotine.gtkgui.utils import human_speed
 from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import save_columns
 from pynicotine.gtkgui.utils import select_user_row_iter
+from pynicotine.gtkgui.utils import set_treeview_selected_row
+from pynicotine.gtkgui.utils import show_file_path_tooltip
+from pynicotine.gtkgui.utils import triggers_context_menu
+from pynicotine.gtkgui.utils import update_widget_visuals
 
 
 class TransferList:
 
-    def __init__(self, frame, widget, type):
+    def __init__(self, frame, type):
+
         self.frame = frame
-        self.widget = widget
         self.type = type
+
+        load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", type + "s.ui"))
+        self.frame.__dict__[type + "svbox"].add(self.Main)
+        self.widget = widget = self.__dict__[type.title() + "List"]
+
         self.last_ui_update = self.last_save = time()
-        self.list = None
+        self.list = []
         self.users = {}
         self.paths = {}
 
@@ -77,10 +91,6 @@ class TransferList:
         self.extension_list_template = _("All %(ext)s")
         self.files_template = _("%(number)2s files ")
 
-        widget.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
-        widget.set_enable_tree_lines(True)
-        widget.set_rubber_banding(True)
-
         self.transfersmodel = Gtk.TreeStore(
             str,                   # (0)  user
             str,                   # (1)  path
@@ -104,56 +114,52 @@ class TransferList:
 
         text_color = self.frame.np.config.sections["ui"]["search"]
 
-        widths = self.frame.np.config.sections["columns"]["{}_widths".format(type)]
         self.cols = cols = initialise_columns(
+            type,
             widget,
-            [_("User"), widths[0], "text", None, (text_color, None)],
-            [_("Path"), widths[1], "text", None, (text_color, None)],
-            [_("Filename"), widths[2], "text", None, (text_color, None)],
-            [_("Status"), widths[3], "text", None, (text_color, None)],
-            [_("Queue Position"), widths[4], "number", None, (text_color, None)],
-            [_("Percent"), widths[5], "progress"],
-            [_("Size"), widths[6], "number", None, (text_color, None)],
-            [_("Speed"), widths[7], "number", None, (text_color, None)],
-            [_("Time elapsed"), widths[8], "number", None, (text_color, None)],
-            [_("Time left"), widths[9], "number", None, (text_color, None)],
+            ["user", _("User"), 200, "text", None, (text_color, None)],
+            ["path", _("Path"), 400, "text", None, (text_color, None)],
+            ["filename", _("Filename"), 400, "text", None, (text_color, None)],
+            ["status", _("Status"), 140, "text", None, (text_color, None)],
+            ["queue_position", _("Queue Position"), 50, "number", None, (text_color, None)],
+            ["percent", _("Percent"), 70, "progress", None, None],
+            ["size", _("Size"), 170, "number", None, (text_color, None)],
+            ["speed", _("Speed"), 90, "number", None, (text_color, None)],
+            ["time_elapsed", _("Time Elapsed"), 140, "number", None, (text_color, None)],
+            ["time_left", _("Time Left"), 140, "number", None, (text_color, None)],
         )
 
-        self.col_user, self.col_path, self.col_filename, self.col_status, self.col_position, self.col_percent, self.col_human_size, self.col_human_speed, self.col_time_elapsed, self.col_time_left = cols
-
-        hide_columns(cols, self.frame.np.config.sections["columns"][self.type + "_columns"])
-
-        self.col_user.set_sort_column_id(0)
-        self.col_path.set_sort_column_id(1)
-        self.col_filename.set_sort_column_id(2)
-        self.col_status.set_sort_column_id(11)
-        self.col_position.set_sort_column_id(17)
-        self.col_percent.set_sort_column_id(5)
-        self.col_human_size.set_sort_column_id(12)
-        self.col_human_speed.set_sort_column_id(14)
-        self.col_time_elapsed.set_sort_column_id(8)
-        self.col_time_left.set_sort_column_id(9)
+        cols["user"].set_sort_column_id(0)
+        cols["path"].set_sort_column_id(1)
+        cols["filename"].set_sort_column_id(2)
+        cols["status"].set_sort_column_id(11)
+        cols["queue_position"].set_sort_column_id(17)
+        cols["percent"].set_sort_column_id(5)
+        cols["size"].set_sort_column_id(12)
+        cols["speed"].set_sort_column_id(14)
+        cols["time_elapsed"].set_sort_column_id(8)
+        cols["time_left"].set_sort_column_id(9)
 
         widget.set_model(self.transfersmodel)
 
-        widget.connect("button_press_event", self.on_popup_menu, "mouse")
-        widget.connect("key-press-event", self.on_key_press_event)
+        self.group_dropdown = frame.__dict__["ToggleTree%ss" % self.type.title()]
+        self.expand_button = frame.__dict__["Expand%ss" % self.type.title()]
 
-        self.update_colours()
+        self.group_dropdown.connect("changed", self.on_toggle_tree)
+        self.group_dropdown.set_active(frame.np.config.sections["transfers"]["group%ss" % self.type])
+
+        self.expand_button.connect("toggled", self.on_expand_tree)
+        self.expand_button.set_active(frame.np.config.sections["transfers"]["%ssexpanded" % self.type])
+
+        self.update_visuals()
 
     def save_columns(self):
-        columns = []
-        widths = []
+        save_columns(self.type, self.widget.get_columns())
 
-        for column in self.widget.get_columns():
-            columns.append(column.get_visible())
-            widths.append(column.get_width())
+    def update_visuals(self):
 
-        self.frame.np.config.sections["columns"][self.type + "_columns"] = columns
-        self.frame.np.config.sections["columns"][self.type + "_widths"] = widths
-
-    def update_colours(self):
-        self.frame.change_list_font(self.widget, self.frame.np.config.sections["ui"]["transfersfont"])
+        for widget in self.__dict__.values():
+            update_widget_visuals(widget, list_font_target="transfersfont")
 
     def init_interface(self, list):
         self.list = list
@@ -162,7 +168,7 @@ class TransferList:
 
     def conn_close(self):
         self.widget.set_sensitive(False)
-        self.list = None
+        self.list = []
         self.clear()
 
     def select_transfers(self):
@@ -172,17 +178,7 @@ class TransferList:
         self.widget.get_selection().selected_foreach(self.selected_transfers_callback)
 
     def new_transfer_notification(self):
-        current_page = self.frame.MainNotebook.get_current_page()
-        my_page = self.frame.MainNotebook.page_num(self.myvbox)
-
-        if (current_page == my_page):
-            return
-
-        tablabel = self.frame.get_tab_label(self.frame.MainNotebook.get_tab_label(self.myvbox))
-        if not tablabel:
-            return
-
-        tablabel.set_image(self.frame.images["online"])
+        self.frame.request_tab_icon(self.tab_label)
 
     def on_ban(self, widget):
         self.select_transfers()
@@ -193,8 +189,8 @@ class TransferList:
     def on_file_search(self, widget):
 
         for transfer in self.selected_transfers:
-            self.frame.search_entry.set_text(transfer.filename.rsplit("\\", 1)[1])
-            self.frame.change_main_page(None, "search")
+            self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
+            self.frame.change_main_page("search")
             break
 
     def rebuild_transfers(self):
@@ -220,11 +216,12 @@ class TransferList:
             iterator = model.iter_next(iterator)
 
     def select_transfer(self, model, iterator, selectuser=False):
+
         user = model.get_value(iterator, 0)
-        file = model.get_value(iterator, 10)
+        filepath = model.get_value(iterator, 10)
 
         for i in self.list:
-            if i.user == user and i.filename == file:
+            if i.user == user and i.filename == filepath:
                 self.selected_transfers.add(i)
                 break
 
@@ -253,13 +250,15 @@ class TransferList:
 
             self.last_save = curtime
 
-        if not forceupdate:
-            current_page = self.frame.MainNotebook.get_current_page()
-            my_page = self.frame.MainNotebook.page_num(self.myvbox)
+        finished = (transfer is not None and transfer.status == "Finished")
 
-            if (current_page != my_page):
-                """ No need to do unnecessary work if transfers are not visible """
-                return
+        if forceupdate or finished or \
+                (curtime - self.last_ui_update) > 1:
+            self.frame.update_bandwidth()
+
+        if not forceupdate and self.frame.current_tab_label != self.tab_label:
+            """ No need to do unnecessary work if transfers are not visible """
+            return
 
         if transfer is not None:
             self.update_specific(transfer)
@@ -268,18 +267,13 @@ class TransferList:
             for i in self.list:
                 self.update_specific(i)
 
-        finished = (transfer is not None and transfer.status == "Finished")
-
-        if not forceupdate and \
-            not finished and \
-                (curtime - self.last_ui_update) <= 1:
+        if forceupdate or finished or \
+                (curtime - self.last_ui_update) > 1:
 
             """ Unless a transfer finishes, use a cooldown to avoid updating
             too often """
 
-            return
-
-        self.update_parent_rows()
+            self.update_parent_rows()
 
     def update_parent_rows(self, only_remove=False):
 
@@ -359,7 +353,9 @@ class TransferList:
             iterator = self.transfersmodel.iter_next(iterator)
 
         if totalsize > 0:
-            percent = ((100 * position) / totalsize)
+            percent = min(((100 * position) / totalsize), 100)
+        else:
+            percent = 100
 
         if speed > 0:
             hspeed = human_speed(speed)
@@ -432,13 +428,11 @@ class TransferList:
 
         try:
             icurrentbytes = int(currentbytes)
-            if icurrentbytes == int(transfer.size):
-                percent = 100
-            else:
-                percent = ((100 * icurrentbytes) / int(size))
+            percent = min(((100 * icurrentbytes) / int(size)), 100)
+
         except Exception:
             icurrentbytes = 0
-            percent = 0
+            percent = 100
 
         # Modify old transfer
         if transfer.iter is not None:
@@ -464,7 +458,7 @@ class TransferList:
             shortfn = fn.split("\\")[-1]
             filecount = 1
 
-            if self.tree_users > 0:
+            if self.tree_users != "ungrouped":
                 # Group by folder or user
 
                 if user not in self.users:
@@ -477,7 +471,7 @@ class TransferList:
 
                 parent = self.users[user]
 
-                if self.tree_users == 1:
+                if self.tree_users == "folder_grouping":
                     # Group by folder
 
                     """ Paths can be empty if files are downloaded individually, make sure we
@@ -503,7 +497,7 @@ class TransferList:
                 parent = None
 
             # Add a new transfer
-            if self.tree_users == 1:
+            if self.tree_users == "folder_grouping":
                 # Group by folder, path not visible
                 path = None
             else:
@@ -517,10 +511,32 @@ class TransferList:
 
             # Expand path
             if parent is not None:
-                path = self.transfersmodel.get_path(iterator)
-                self.expand(path)
+                transfer_path = self.transfersmodel.get_path(iterator)
+
+                if self.tree_users == "folder_grouping":
+                    # Group by folder, we need the user path to expand it
+
+                    user_path = self.transfersmodel.get_path(self.users[user])
+                else:
+                    user_path = None
+
+                self.expand(transfer_path, user_path)
+
+    def abort_transfers(self, remove_file=False, clear=False):
+
+        for i in self.selected_transfers:
+            if i.status != "Finished":
+                self.frame.np.transfers.abort_transfer(i, remove_file)
+
+                if not clear:
+                    i.status = "Aborted"
+                    self.update(i)
+
+            if clear:
+                self.remove_specific(i)
 
     def remove_specific(self, transfer, cleartreeviewonly=False):
+
         if not cleartreeviewonly:
             self.list.remove(transfer)
 
@@ -529,7 +545,20 @@ class TransferList:
 
         self.update_parent_rows(only_remove=True)
 
+    def clear_transfers(self, status):
+
+        for i in self.list[:]:
+            if i.status in status:
+                if i.transfertimer is not None:
+                    i.transfertimer.cancel()
+
+                if i.status == "Queued":
+                    self.frame.np.transfers.abort_transfer(i)
+
+                self.remove_specific(i)
+
     def clear(self):
+
         self.users.clear()
         self.paths.clear()
         self.selected_transfers = set()
@@ -540,6 +569,60 @@ class TransferList:
             for i in self.list:
                 i.iter = None
 
+    def expand(self, transfer_path, user_path):
+
+        if self.expand_button.get_active():
+            self.widget.expand_to_path(transfer_path)
+
+        elif user_path and self.tree_users == "folder_grouping":
+            # Group by folder, show user folders in collapsed mode
+
+            self.widget.expand_to_path(user_path)
+
+    def on_expand_tree(self, widget):
+
+        expand_button_icon = self.frame.__dict__["Expand%ssImage" % self.type.title()]
+        expanded = self.expand_button.get_active()
+
+        if expanded:
+            self.widget.expand_all()
+            expand_button_icon.set_from_icon_name("go-up-symbolic", Gtk.IconSize.BUTTON)
+        else:
+            collapse_treeview(self.widget, self.tree_users)
+            expand_button_icon.set_from_icon_name("go-down-symbolic", Gtk.IconSize.BUTTON)
+
+        self.frame.np.config.sections["transfers"]["%ssexpanded" % self.type] = expanded
+        self.frame.np.config.write_configuration()
+
+    def on_toggle_tree(self, widget):
+
+        pos = self.group_dropdown.get_active()
+        self.frame.np.config.sections["transfers"]["group%ss" % self.type] = pos
+
+        self.tree_users = self.group_dropdown.get_active_id()
+
+        if self.tree_users == "ungrouped":
+            self.expand_button.hide()
+        else:
+            self.expand_button.show()
+
+        self.rebuild_transfers()
+
+    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
+        return show_file_path_tooltip(widget, x, y, tooltip, 10)
+
+    def on_list_clicked(self, widget, event):
+
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_menu(widget)
+
+        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
+            self.double_click(event)
+            return True
+
+        return False
+
     def on_popup_menu_users(self, widget):
 
         self.popup_menu_users.clear()
@@ -551,19 +634,9 @@ class TransferList:
             for user in self.selected_users:
 
                 popup = PopupMenu(self.frame, False)
-                popup.setup(
-                    ("#" + _("Send _message"), popup.on_send_message),
-                    ("#" + _("Show IP a_ddress"), popup.on_show_ip_address),
-                    ("#" + _("Get user i_nfo"), popup.on_get_user_info),
-                    ("#" + _("Brow_se files"), popup.on_browse_user),
-                    ("#" + _("Gi_ve privileges"), popup.on_give_privileges),
-                    ("", None),
-                    ("$" + _("_Add user to list"), popup.on_add_to_list),
-                    ("$" + _("_Ban this user"), popup.on_ban_user),
-                    ("$" + _("_Ignore this user"), popup.on_ignore_user),
-                    ("#" + _("Select User's Transfers"), self.on_select_user_transfers)
-                )
-                popup.set_user(user)
+                popup.setup_user_menu(user)
+                popup.append_item(("", None))
+                popup.append_item(("#" + _("Select User's Transfers"), self.on_select_user_transfers))
 
                 items.append((1, user, popup, self.on_popup_menu_user, popup))
 
@@ -576,26 +649,7 @@ class TransferList:
         if popup is None:
             return
 
-        menu = popup
-        user = menu.user
-        items = menu.get_children()
-
-        act = False
-        if len(self.selected_users) >= 1:
-            act = True
-
-        items[0].set_sensitive(act)
-        items[1].set_sensitive(act)
-        items[2].set_sensitive(act)
-        items[3].set_sensitive(act)
-
-        items[6].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-        items[7].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-        items[8].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-
-        for i in range(4, 9):
-            items[i].set_sensitive(act)
-
+        popup.toggle_user_items()
         return True
 
     def on_select_user_transfers(self, widget):
@@ -615,6 +669,19 @@ class TransferList:
 
         self.select_transfers()
 
+    def on_play_files(self, widget, prefix=""):
+        _thread.start_new_thread(self._on_play_files, (widget, prefix))
+
+    def on_copy_file_path(self, widget):
+
+        if not self.selected_transfers:
+            return
+
+        i = next(iter(self.selected_transfers))
+        text = self.transfersmodel.get_value(i.iter, 10)
+
+        self.frame.clip.set_text(text, -1)
+
     def on_copy_url(self, widget):
         i = next(iter(self.selected_transfers))
         self.frame.set_clipboard_url(i.user, i.filename)
@@ -629,22 +696,9 @@ class TransferList:
 
         self.frame.set_clipboard_url(i.user, path)
 
-    def on_abort_transfer(self, widget, remove=False, clear=False):
-
-        for i in self.selected_transfers:
-
-            if i.status != "Finished":
-                self.frame.np.transfers.abort_transfer(i, remove)
-
-                if not clear:
-                    i.status = "Aborted"
-                    self.update(i)
-
-            if clear:
-                self.remove_specific(i)
-
     def on_clear_transfer(self, widget):
-        self.on_abort_transfer(widget, False, True)
+        self.select_transfers()
+        self.abort_transfers(remove_file=False, clear=True)
 
     def on_clear_response(self, dialog, response, data=None):
         if response == Gtk.ResponseType.OK:
@@ -652,40 +706,20 @@ class TransferList:
 
         dialog.destroy()
 
-    def clear_transfers(self, status):
-
-        for i in self.list[:]:
-            if i.status in status:
-                if i.transfertimer is not None:
-                    i.transfertimer.cancel()
-                self.remove_specific(i)
-
     def on_clear_finished(self, widget):
         self.clear_transfers(["Finished"])
 
     def on_clear_aborted(self, widget):
-        statuslist = ["Aborted", "Cancelled"]
-        self.clear_transfers(statuslist)
+        self.clear_transfers(["Aborted", "Cancelled"])
 
     def on_clear_filtered(self, widget):
-        statuslist = ["Filtered"]
-        self.clear_transfers(statuslist)
-
-    def on_clear_failed(self, widget):
-        statuslist = ["Cannot connect", "Connection closed by peer", "Local file error", "Remote file error", "Getting address", "Waiting for peer to connect", "Initializing transfer"]
-        self.clear_transfers(statuslist)
+        self.clear_transfers(["Filtered"])
 
     def on_clear_paused(self, widget):
-        statuslist = ["Paused"]
-        self.clear_transfers(statuslist)
+        self.clear_transfers(["Paused"])
 
     def on_clear_finished_aborted(self, widget):
-        statuslist = ["Aborted", "Cancelled", "Finished", "Filtered"]
-        self.clear_transfers(statuslist)
+        self.clear_transfers(["Aborted", "Cancelled", "Finished", "Filtered"])
 
     def on_clear_finished_erred(self, widget):
-        statuslist = ["Aborted", "Cancelled", "Finished", "Filtered", "Cannot connect", "Connection closed by peer", "Local file error", "Remote file error"]
-        self.clear_transfers(statuslist)
-
-    def on_clear_queued(self, widget):
-        self.clear_transfers(["Queued"])
+        self.clear_transfers(["Aborted", "Cancelled", "Finished", "Filtered", "Cannot connect", "Connection closed by peer", "Local file error", "Remote file error"])
