@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2009 Quinox <quinox@users.sf.net>
@@ -23,7 +23,6 @@
 
 import os
 import time
-from gettext import gettext as _
 
 from gi.repository import Gdk
 from gi.repository import GLib
@@ -32,12 +31,16 @@ from gi.repository import Gtk
 
 from pynicotine import slskmessages
 from pynicotine.gtkgui.dialogs import entry_dialog
-from pynicotine.gtkgui.utils import hide_columns
 from pynicotine.gtkgui.utils import humanize
 from pynicotine.gtkgui.utils import human_speed
 from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import save_columns
+from pynicotine.gtkgui.utils import set_treeview_selected_row
 from pynicotine.gtkgui.utils import show_country_tooltip
+from pynicotine.gtkgui.utils import triggers_context_menu
+from pynicotine.gtkgui.utils import update_widget_visuals
 from pynicotine.logfacility import log
 
 
@@ -49,23 +52,7 @@ class UserList:
         self.frame = frame
         config = self.frame.np.config.sections
 
-        builder = Gtk.Builder()
-
-        builder.set_translation_domain('nicotine')
-        builder.add_from_file(os.path.join(os.path.dirname(os.path.realpath(__file__)), "ui", "buddylist.ui"))
-
-        self.temp_window = builder.get_object("TempWindow")
-
-        for i in builder.get_objects():
-            try:
-                self.__dict__[Gtk.Buildable.get_name(i)] = i
-            except TypeError:
-                pass
-
-        self.temp_window.remove(self.userlistvbox)
-        self.temp_window.destroy()
-
-        builder.connect_signals(self)
+        load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "buddylist.ui"))
 
         """ Columns """
 
@@ -87,53 +74,49 @@ class UserList:
             str                   # (14) country
         )
 
-        widths = self.frame.np.config.sections["columns"]["userlist_widths"]
         self.cols = cols = initialise_columns(
+            "buddy_list",
             self.UserListTree,
-            [_("Status"), widths[0], "pixbuf"],
-            [_("Country"), widths[1], "pixbuf"],
-            [_("User"), widths[2], "text", self.cell_data_func],
-            [_("Speed"), widths[3], "number", self.cell_data_func],
-            [_("Files"), widths[4], "number", self.cell_data_func],
-            [_("Trusted"), widths[5], "toggle"],
-            [_("Notify"), widths[6], "toggle"],
-            [_("Privileged"), widths[7], "toggle"],
-            [_("Last seen"), widths[8], "text", self.cell_data_func],
-            [_("Comments"), widths[9], "edit", self.cell_data_func]
+            ["status", _("Status"), 25, "pixbuf", None, None],
+            ["country", _("Country"), 25, "pixbuf", None, None],
+            ["user", _("User"), 250, "text", None, None],
+            ["speed", _("Speed"), 150, "number", None, None],
+            ["files", _("Files"), 150, "number", None, None],
+            ["trusted", _("Trusted"), 0, "toggle", None, None],
+            ["notify", _("Notify"), 0, "toggle", None, None],
+            ["privileged", _("Privileged"), 0, "toggle", None, None],
+            ["last_seen", _("Last seen"), 160, "text", None, None],
+            ["comments", _("Comments"), 400, "edit", None, None]
         )
 
-        self.col_status, self.col_country, self.col_user, self.col_speed, self.col_files, self.col_trusted, self.col_notify, self.col_privileged, self.col_last_seen, self.col_comments = cols
+        cols["status"].set_sort_column_id(10)
+        cols["country"].set_sort_column_id(14)
+        cols["user"].set_sort_column_id(2)
+        cols["speed"].set_sort_column_id(11)
+        cols["files"].set_sort_column_id(12)
+        cols["trusted"].set_sort_column_id(5)
+        cols["notify"].set_sort_column_id(6)
+        cols["privileged"].set_sort_column_id(7)
+        cols["last_seen"].set_sort_column_id(13)
+        cols["comments"].set_sort_column_id(9)
 
-        hide_columns(cols, config["columns"]["userlist"])
-
-        self.col_status.set_sort_column_id(10)
-        self.col_country.set_sort_column_id(14)
-        self.col_user.set_sort_column_id(2)
-        self.col_speed.set_sort_column_id(11)
-        self.col_files.set_sort_column_id(12)
-        self.col_trusted.set_sort_column_id(5)
-        self.col_notify.set_sort_column_id(6)
-        self.col_privileged.set_sort_column_id(7)
-        self.col_last_seen.set_sort_column_id(13)
-        self.col_comments.set_sort_column_id(9)
-
-        self.col_status.get_widget().hide()
-        self.col_country.get_widget().hide()
+        cols["status"].get_widget().hide()
+        cols["country"].get_widget().hide()
 
         if config["columns"]["hideflags"]:
-            cols[1].set_visible(0)
-            config["columns"]["userlist"][1] = 0
+            cols["country"].set_visible(0)
+            config["columns"]["buddy_list"]["country"]["visible"] = 0
 
-        for render in self.col_trusted.get_cells():
+        for render in cols["trusted"].get_cells():
             render.connect('toggled', self.cell_toggle_callback, self.UserListTree, 5)
 
-        for render in self.col_notify.get_cells():
+        for render in cols["notify"].get_cells():
             render.connect('toggled', self.cell_toggle_callback, self.UserListTree, 6)
 
-        for render in self.col_privileged.get_cells():
+        for render in cols["privileged"].get_cells():
             render.connect('toggled', self.cell_toggle_callback, self.UserListTree, 7)
 
-        for render in self.col_comments.get_cells():
+        for render in cols["comments"].get_cells():
             render.connect('edited', self.cell_edited_callback, self.UserListTree, 9)
 
         self.UserListTree.set_model(self.usersmodel)
@@ -141,7 +124,11 @@ class UserList:
         """ Buddy list """
 
         for user in self.frame.np.config.sections["server"]["userlist"]:
-            username, comment, notify, privileged, trusted, last_seen, country = user
+            try:
+                username, comment, notify, privileged, trusted, last_seen, country = user
+            except ValueError:
+                # Invalid user row
+                continue
 
             try:
                 time_from_epoch = time.mktime(time.strptime(last_seen, "%m/%d/%Y %H:%M:%S"))
@@ -171,74 +158,60 @@ class UserList:
 
         self.usersmodel.set_sort_column_id(2, Gtk.SortType.ASCENDING)
 
+        self.buddies_combo_entries = (
+            self.frame.UserSearchCombo, self.frame.PrivateChatCombo, self.frame.UserInfoCombo, self.frame.UserBrowseCombo
+        )
+
+        self.buddies_combos_fill()
+
         """ Popup """
 
         self.popup_menu_private_rooms = PopupMenu(self.frame, False)
         self.popup_menu = popup = PopupMenu(frame)
+        popup.setup_user_menu()
+        popup.get_items()[_("_Add User To List")].set_visible(False)
 
-        popup.setup(
-            ("#" + _("Send _message"), popup.on_send_message),
-            ("", None),
-            ("#" + _("Show IP a_ddress"), popup.on_show_ip_address),
-            ("#" + _("Get user i_nfo"), popup.on_get_user_info),
-            ("#" + _("Brow_se files"), popup.on_browse_user),
-            ("#" + _("Gi_ve privileges"), popup.on_give_privileges),
-            ("$" + _("_Ban this user"), popup.on_ban_user),
-            ("$" + _("_Ignore this user"), popup.on_ignore_user),
-            ("", None),
-            ("$" + _("_Online notify"), self.on_notify),
-            ("$" + _("_Privileged"), self.on_privileged),
-            ("$" + _("_Trusted"), self.on_trusted),
-            ("", None),
-            ("#" + _("Edit _comments"), self.on_edit_comments),
-            ("#" + _("_Remove"), self.on_remove_user),
-            (1, _("Private rooms"), self.popup_menu_private_rooms, popup.on_private_rooms, self.popup_menu_private_rooms)
-        )
+        popup.append_item(("", None))
+        popup.append_item(("$" + _("_Online Notify"), self.on_notify))
+        popup.append_item(("$" + _("_Privileged"), self.on_privileged))
+        popup.append_item(("$" + _("_Trusted"), self.on_trusted))
+        popup.append_item(("", None))
+        popup.append_item((1, _("Private Rooms"), self.popup_menu_private_rooms, popup.on_private_rooms, self.popup_menu_private_rooms))
+        popup.append_item(("#" + _("Edit _Comments"), self.on_edit_comments))
+        popup.append_item(("#" + _("_Remove"), self.on_remove_user))
 
-        items = self.popup_menu.get_children()
-        self.menu_send_message = items[0]
-        self.menu_show_ip_address = items[2]
-        self.menu_get_user_info = items[3]
-        self.menu_browse_user = items[4]
-        self.menu_give_privileges = items[5]
-        self.menu_ban_user = items[6]
-        self.menu_ignore_user = items[7]
-        self.menu_on_notify = items[9]
-        self.menu_on_privileged = items[10]
-        self.menu_on_trusted = items[11]
-        self.menu_edit_comments = items[13]
-        self.menu_remove_user = items[14]
-        self.menu_private_rooms = items[15]
+        self.update_visuals()
 
-        self.UserListTree.connect("button_press_event", self.on_popup_menu)
+    def buddies_combos_fill(self):
+
+        for widget in self.buddies_combo_entries:
+            widget.remove_all()
+            widget.append_text("")
+
+            for user in self.frame.np.config.sections["server"]["userlist"]:
+                widget.append_text(user[0])
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
-        return show_country_tooltip(widget, x, y, tooltip, 14, 'flag_')
+        return show_country_tooltip(widget, x, y, tooltip, 14)
 
-    def on_add_user(self, widget):
+    def on_add_user(self, widget, *args):
+
+        text = widget.get_text()
 
-        text = self.AddUserEntry.get_text()
         if not text:
             return
 
-        self.AddUserEntry.set_text("")
+        widget.set_text("")
         self.add_to_list(text)
 
-    def update_colours(self):
-        self.frame.set_text_bg(self.AddUserEntry)
+    def update_visuals(self):
+
+        for widget in self.__dict__.values():
+            update_widget_visuals(widget)
 
     def on_settings_ban_ignore(self, widget):
         self.frame.on_settings_ban_ignore(widget)
 
-    def cell_data_func(self, column, cellrenderer, model, iterator, dummy="dummy"):
-
-        colour = self.frame.np.config.sections["ui"]["search"]
-
-        if colour == "":
-            colour = None
-
-        cellrenderer.set_property("foreground", colour)
-
     def cell_toggle_callback(self, widget, index, treeview, pos):
 
         iterator = self.usersmodel.get_iter(index)
@@ -304,42 +277,69 @@ class UserList:
                 user = i[2]
                 self.set_last_seen(user)
 
-    def on_popup_menu(self, widget, event):
+    def get_selected_username(self, treeview):
 
-        d = self.UserListTree.get_path_at_pos(int(event.x), int(event.y))
+        model, iterator = treeview.get_selection().get_selected()
 
-        if d:
-            path, column, x, y = d
-            model = self.UserListTree.get_model()
-            iterator = model.get_iter(path)
-            user = model.get_value(iterator, 2)
+        if iterator is None:
+            return None
+
+        return model.get_value(iterator, 2)
+
+    def get_selected_username_details(self, treeview):
+
+        model, iterator = treeview.get_selection().get_selected()
+
+        if iterator is not None:
+            username = model.get_value(iterator, 2)
+            trusted = model.get_value(iterator, 5)
+            notify = model.get_value(iterator, 6)
+            privileged = model.get_value(iterator, 7)
             status = model.get_value(iterator, 10)
 
-            if event.button != 3:
-                if event.type == Gdk.EventType._2BUTTON_PRESS:
-                    self.frame.privatechats.send_message(user, None, 1)
-                    self.frame.change_main_page(None, "private")
-                return
+        else:
+            username = trusted = notify = privileged = status = None
 
-            self.popup_menu.set_user(user)
+        return username, trusted, notify, privileged, status
 
-            self.menu_send_message.set_sensitive(status)
-            self.menu_show_ip_address.set_sensitive(status)
-            self.menu_get_user_info.set_sensitive(status)
-            self.menu_browse_user.set_sensitive(status)
-            self.menu_give_privileges.set_sensitive(status)
-            self.menu_private_rooms.set_sensitive(
-                status or
-                self.popup_menu.user != self.frame.np.config.sections["server"]["login"]
-            )
+    def on_list_clicked(self, widget, event):
 
-            self.menu_ban_user.set_active(user in self.frame.np.config.sections["server"]["banlist"])
-            self.menu_ignore_user.set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-            self.menu_on_notify.set_active(model.get_value(iterator, 6))
-            self.menu_on_privileged.set_active(model.get_value(iterator, 7))
-            self.menu_on_trusted.set_active(model.get_value(iterator, 5))
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_menu(widget)
 
-            self.popup_menu.popup(None, None, None, None, event.button, event.time)
+        if event.type == Gdk.EventType._2BUTTON_PRESS:
+            user = self.get_selected_username(widget)
+
+            if user is not None:
+                self.frame.privatechats.send_message(user, show_user=True)
+                self.frame.change_main_page("private")
+                return True
+
+        return False
+
+    def on_popup_menu(self, widget):
+
+        username, trusted, notify, privileged, status = self.get_selected_username_details(widget)
+        if username is None:
+            return False
+
+        self.popup_menu.set_user(username)
+        self.popup_menu.toggle_user_items()
+
+        items = self.popup_menu.get_items()
+
+        items[_("Private Rooms")].set_sensitive(
+            status or
+            self.popup_menu.user != self.frame.np.config.sections["server"]["login"]
+        )
+
+        items[_("_Online Notify")].set_active(notify)
+        items[_("_Privileged")].set_active(privileged)
+        items[_("_Trusted")].set_active(trusted)
+
+        self.popup_menu.popup()
+        return True
 
     def get_iter(self, user):
 
@@ -411,7 +411,7 @@ class UserList:
         if iterator is None:
             return
 
-        if user not in [i[2] for i in self.usersmodel]:
+        if user not in (i[2] for i in self.usersmodel):
             return
 
         self.usersmodel.set(
@@ -422,7 +422,7 @@ class UserList:
 
     def add_to_list(self, user):
 
-        if user in [i[2] for i in self.usersmodel]:
+        if user in (i[2] for i in self.usersmodel):
             return
 
         row = [self.frame.get_status_image(0), None, user, "", "", False, False, False, _("Never seen"), "", 0, 0, 0, 0, ""]
@@ -432,11 +432,11 @@ class UserList:
         self.frame.np.queue.put(slskmessages.AddUser(user))
         self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
 
-        for widget in self.frame.buddies_combo_entries:
-            GLib.idle_add(widget.append, user)
+        for widget in self.buddies_combo_entries:
+            widget.append_text(user)
 
         if self.frame.np.config.sections["words"]["buddies"]:
-            GLib.idle_add(self.frame.chatrooms.roomsctrl.update_completions)
+            GLib.idle_add(self.frame.chatrooms.update_completions)
             GLib.idle_add(self.frame.privatechats.update_completions)
 
     def on_edit_comments(self, widget):
@@ -473,16 +473,7 @@ class UserList:
         self.frame.np.config.write_configuration()
 
     def save_columns(self):
-
-        columns = []
-        widths = []
-
-        for column in self.UserListTree.get_columns():
-            columns.append(column.get_visible())
-            widths.append(column.get_width())
-
-        self.frame.np.config.sections["columns"]["userlist"] = columns
-        self.frame.np.config.sections["columns"]["userlist_widths"] = widths
+        save_columns("buddy_list", self.UserListTree.get_columns())
 
     def remove_from_list(self, user):
 
@@ -493,11 +484,10 @@ class UserList:
 
         self.save_user_list()
 
-        for widget in self.frame.buddies_combo_entries:
-            GLib.idle_add(widget.remove, user)
+        self.buddies_combos_fill()
 
         if self.frame.np.config.sections["words"]["buddies"]:
-            GLib.idle_add(self.frame.chatrooms.roomsctrl.update_completions)
+            GLib.idle_add(self.frame.chatrooms.update_completions)
             GLib.idle_add(self.frame.privatechats.update_completions)
 
     def on_remove_user(self, widget):
