@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2016-2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
@@ -23,14 +23,14 @@
 
 import os
 
-from gettext import gettext as _
-
 from gi.repository import GLib
-from gi.repository import GObject
 from gi.repository import Gtk
 
 from pynicotine import slskmessages
+from pynicotine.gtkgui.dialogs import option_dialog
 from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import load_ui_elements
+from pynicotine.gtkgui.utils import update_widget_visuals
 from pynicotine.logfacility import log
 
 
@@ -45,50 +45,33 @@ class WishList:
         self.timer = None
         self.wishes = {}
 
-        builder = Gtk.Builder()
+        load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "dialogs", "wishlist.ui"))
+        self.WishListDialog.set_transient_for(frame.MainWindow)
 
-        builder.set_translation_domain('nicotine')
-        builder.add_from_file(os.path.join(os.path.dirname(os.path.realpath(__file__)), "ui", "wishlist.ui"))
-
-        self.wish_list_dialog = builder.get_object("WishListDialog")
-        builder.connect_signals(self)
-
-        for i in builder.get_objects():
-            try:
-                self.__dict__[Gtk.Buildable.get_name(i)] = i
-            except TypeError:
-                pass
-
-        self.wish_list_dialog.set_transient_for(frame.MainWindow)
-
-        self.wish_list_dialog.connect("destroy", self.quit)
-        self.wish_list_dialog.connect("destroy-event", self.quit)
-        self.wish_list_dialog.connect("delete-event", self.quit)
-        self.wish_list_dialog.connect("delete_event", self.quit)
-
-        self.store = Gtk.ListStore(GObject.TYPE_STRING)
+        self.store = Gtk.ListStore(str)
 
         cols = initialise_columns(
+            None,
             self.WishlistView,
-            [_("Wishes"), -1, "text"]
+            ["wishes", _("Wishes"), -1, "text", None, None]
         )
 
         self.WishlistView.set_model(self.store)
-        self.WishlistView.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
 
         self.store.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         for wish in self.frame.np.config.sections["server"]["autosearch"]:
             self.wishes[wish] = self.store.append([wish])
 
-        renderers = cols[0].get_cells()
+        renderers = cols["wishes"].get_cells()
         for render in renderers:
             render.set_property('editable', True)
             render.connect('edited', self.cell_edited_callback, self.WishlistView, 0)
 
-        self.frame.WishList.connect("clicked", self.show)
+        frame.WishList.connect("clicked", self.show)
 
     def cell_edited_callback(self, widget, index, value, treeview, pos):
+
         store = treeview.get_model()
         iterator = store.get_iter(index)
         old_value = store.get_value(iterator, 0)
@@ -97,14 +80,16 @@ class WishList:
             self.remove_wish(old_value)
             self.add_wish(value)
 
-    def on_add_wish(self, widget):
-        wish = self.AddWishEntry.get_text()
-        self.AddWishEntry.set_text("")
+    def on_add_wish(self, widget, *args):
+
+        wish = widget.get_text()
+        widget.set_text("")
 
         if self.add_wish(wish):
             self.do_wishlist_search(self.searches.searchid, wish)
 
     def on_remove_wish(self, widget):
+
         iters = []
         self.WishlistView.get_selection().selected_foreach(self._remove_wish, iters)
 
@@ -115,10 +100,24 @@ class WishList:
     def _remove_wish(self, model, path, iterator, line):
         line.append(iterator)
 
-    def on_select_all_wishes(self, widget):
-        self.WishlistView.get_selection().select_all()
+    def clear_wishlist_response(self, dialog, response, data):
+
+        if response == Gtk.ResponseType.OK:
+            for wish in self.wishes.copy():
+                self.remove_wish(wish)
+
+        dialog.destroy()
+
+    def on_clear_wishlist(self, *args):
+        option_dialog(
+            parent=self.WishListDialog,
+            title=_('Clear Wishlist?'),
+            message=_('Are you sure you wish to clear your wishlist?'),
+            callback=self.clear_wishlist_response
+        )
 
     def add_wish(self, wish):
+
         if not wish:
             return False
 
@@ -126,7 +125,15 @@ class WishList:
             self.wishes[wish] = self.store.append([wish])
 
         self.searches.searchid += 1
-        self.searches.searches[self.searches.searchid] = [self.searches.searchid, wish, None, 0, True, False]
+
+        self.searches.searches[self.searches.searchid] = {
+            "id": self.searches.searchid,
+            "term": wish,
+            "tab": None,
+            "mode": "wishlist",
+            "remember": True,
+            "ignore": True
+        }
 
         if wish not in self.frame.np.config.sections["server"]["autosearch"]:
             self.frame.np.config.sections["server"]["autosearch"].append(wish)
@@ -134,6 +141,7 @@ class WishList:
         return True
 
     def remove_wish(self, wish):
+
         if wish in self.wishes:
             self.store.remove(self.wishes[wish])
             del self.wishes[wish]
@@ -144,26 +152,34 @@ class WishList:
 
             for number, search in self.searches.searches.items():
 
-                if search[1] == wish and search[4]:
+                if search["term"] == wish and search["remember"]:
 
-                    if search[2] is not None and search[2].showtab:  # Tab visible
-                        search[4] = False
+                    if search["tab"] is not None and search["tab"].showtab:  # Tab visible
+                        search["remember"] = False
                         self.searches.searches[number] = search
 
-                        search[2].RememberCheckButton.set_active(False)
+                        search["tab"].RememberCheckButton.set_active(False)
                     else:
                         del self.searches.searches[number]
 
                     break
 
     def set_interval(self, msg):
+
         self.interval = msg.seconds
 
         if not self.disconnected:
             # Create wishlist searches (without tabs)
             for term in self.frame.np.config.sections["server"]["autosearch"]:
-                self.searches.searches[self.searches.searchid] = [self.searches.searchid, term, None, 0, True, False]
-                self.searches.searchid = (self.searches.searchid + 1) % (2**31)
+                self.searches.searches[self.searches.searchid] = {
+                    "id": self.searches.searchid,
+                    "term": term,
+                    "tab": None,
+                    "mode": "wishlist",
+                    "remember": True,
+                    "ignore": True
+                }
+                self.searches.searchid += 1
 
         self.on_auto_search()
         self.timer = GLib.timeout_add(self.interval * 1000, self.on_auto_search)
@@ -188,8 +204,10 @@ class WishList:
         searches.insert(0, term)
 
         for i in self.searches.searches.values():
-            if i[1] == term and i[4]:
-                self.do_wishlist_search(i[0], term)
+            if i["term"] == term and i["remember"]:
+                i["ignore"] = False
+
+                self.do_wishlist_search(i["id"], term)
                 break
 
         return True
@@ -201,9 +219,14 @@ class WishList:
             GLib.source_remove(self.timer)
             self.timer = None
 
+    def update_visuals(self):
+
+        for widget in self.__dict__.values():
+            update_widget_visuals(widget)
+
     def show(self, widget):
-        self.wish_list_dialog.show()
+        self.WishListDialog.show()
 
     def quit(self, widget, event):
-        self.wish_list_dialog.hide()
+        self.WishListDialog.hide()
         return True
