@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
@@ -22,9 +22,9 @@
 
 import os
 import sys
+import _thread
 
 from ast import literal_eval
-from gettext import gettext as _
 from time import time
 
 from pynicotine import slskmessages
@@ -57,7 +57,7 @@ class PluginHandler(object):
             pass
 
         # Load system-wide plugins
-        prefix = os.path.dirname(__file__)
+        prefix = os.path.dirname(os.path.realpath(__file__))
         self.plugindirs.append(os.path.join(prefix, "plugins"))
 
         # Load home directory plugins
@@ -81,32 +81,42 @@ class PluginHandler(object):
         on = pluginname in self.enabled_plugins
 
         if on:
-            self.disable_plugin(pluginname)
+            _thread.start_new_thread(self.disable_plugin, (pluginname,))
         else:
-            self.enable_plugin(pluginname)
+            _thread.start_new_thread(self.enable_plugin, (pluginname,))
 
     def load_plugin(self, pluginname):
-        path = self.__findplugin(pluginname)
 
-        if path is None:
-            log.add(_("Failed to load plugin '%s', could not find it."), pluginname)
-            return False
+        try:
+            # Import builtin plugin
+            from importlib import import_module
+            plugin = import_module("pynicotine.plugins." + pluginname)
 
-        sys.path.insert(0, path)
+        except Exception:
+            # Import user plugin
+            path = self.__findplugin(pluginname)
 
-        from importlib.machinery import SourceFileLoader
-        plugin = SourceFileLoader(pluginname, os.path.join(path, '__init__.py')).load_module()
+            if path is None:
+                log.add(_("Failed to load plugin '%s', could not find it."), pluginname)
+                return False
+
+            from importlib.machinery import SourceFileLoader
+            plugin = SourceFileLoader(pluginname, os.path.join(path, '__init__.py')).load_module()
 
         instance = plugin.Plugin(self)
-        self.plugin_settings(instance)
+        self.plugin_settings(pluginname, instance)
         instance.LoadNotification()
 
-        sys.path = sys.path[1:]
-
         self.loaded_plugins[pluginname] = plugin
         return plugin
 
     def enable_plugin(self, pluginname):
+
+        # Our config file doesn't play nicely with some characters
+        if "=" in pluginname:
+            log.add(_("Unable to enable plugin %(name)s. Plugin folder name contains invalid characters: %(characters)s") % {"name": pluginname, "characters": "="})
+            return
+
         if pluginname in self.enabled_plugins:
             return
 
@@ -192,21 +202,21 @@ class PluginHandler(object):
         to_enable = self.config.sections["plugins"]["enabled"]
 
         for plugin in to_enable:
-            self.enable_plugin(plugin)
+            _thread.start_new_thread(self.enable_plugin, (plugin,))
 
-    def plugin_settings(self, plugin):
+    def plugin_settings(self, pluginname, plugin):
         try:
             if not hasattr(plugin, "settings"):
                 return
 
-            if plugin.__id__ not in self.config.sections["plugins"]:
-                self.config.sections["plugins"][plugin.__id__] = plugin.settings
+            if pluginname not in self.config.sections["plugins"]:
+                self.config.sections["plugins"][pluginname] = plugin.settings
 
             for i in plugin.settings:
-                if i not in self.config.sections["plugins"][plugin.__id__]:
-                    self.config.sections["plugins"][plugin.__id__][i] = plugin.settings[i]
+                if i not in self.config.sections["plugins"][pluginname]:
+                    self.config.sections["plugins"][pluginname][i] = plugin.settings[i]
 
-            customsettings = self.config.sections["plugins"][plugin.__id__]
+            customsettings = self.config.sections["plugins"][pluginname]
 
             for key in customsettings:
                 if key in plugin.settings:
@@ -347,8 +357,8 @@ class PluginHandler(object):
     def outgoing_buddy_search_event(self, text):
         return self.trigger_event("OutgoingBuddySearchEvent", (text,))
 
-    def outgoing_user_search_event(self, users):
-        return self.trigger_event("OutgoingUserSearchEvent", (users,))
+    def outgoing_user_search_event(self, users, text):
+        return self.trigger_event("OutgoingUserSearchEvent", (users, text))
 
     def user_resolve_notification(self, user, ip, port, country=None):
         """Notification for user IP:Port resolving.
@@ -488,19 +498,15 @@ class BasePlugin(object):
         # Never override this function, override init() instead
         self.parent = parent
         self.frame = parent.frame
-        try:
-            self.__id__
-        except AttributeError:
-            # See http://docs.python.org/library/configparser.html
-            # %(name)s will lead to replacements so we need to filter out those symbols.
-            self.__id__ = self.__name__.lower().replace(' ', '_').replace('%', '_').replace('=', '_')
+
         self.init()
         for (trigger, func) in self.__publiccommands__:
-            self.frame.chatrooms.roomsctrl.CMDS.add('/' + trigger + ' ')
+            self.frame.chatrooms.CMDS.add('/' + trigger + ' ')
         for (trigger, func) in self.__privatecommands__:
             self.frame.privatechats.CMDS.add('/' + trigger + ' ')
 
     def init(self):
+        # Custom init function for plugins
         pass
 
     def LoadSettings(self, settings):  # noqa
@@ -551,7 +557,7 @@ class BasePlugin(object):
     def OutgoingBuddySearchEvent(self, text):  # noqa
         pass
 
-    def OutgoingUserSearchEvent(self, users):  # noqa
+    def OutgoingUserSearchEvent(self, users, text):  # noqa
         pass
 
     def UserResolveNotification(self, user, ip, port, country):  # noqa
@@ -593,7 +599,7 @@ class BasePlugin(object):
 
     def fakepublic(self, room, user, text):
         try:
-            room = self.frame.chatrooms.roomsctrl.joinedrooms[room]
+            room = self.frame.chatrooms.joinedrooms[room]
         except KeyError:
             return False
 
