@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2020 Mathias <mail@mathias.is>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
@@ -35,17 +35,16 @@ import threading
 import time
 import _thread
 
-from gettext import gettext as _
-
 from pynicotine import slskmessages
 from pynicotine import slskproto
 from pynicotine import transfers
 from pynicotine.config import Config
 from pynicotine.geoip.ip2location import IP2Location
 from pynicotine.logfacility import log
-from pynicotine.pluginsystem import PluginHandler
+from pynicotine.nowplaying import NowPlaying
 from pynicotine.shares import Shares
 from pynicotine.slskmessages import new_id
+from pynicotine.transfers import Statistics
 from pynicotine.utils import clean_file
 from pynicotine.utils import unescape
 
@@ -99,7 +98,7 @@ class ConnectToPeerTimeout(Timeout):
 class NetworkEventProcessor:
     """ This class contains handlers for various messages from the networking thread """
 
-    def __init__(self, ui_callback, network_callback, setstatus, bindip, port, data_dir, config, plugins):
+    def __init__(self, ui_callback, network_callback, setstatus, bindip, port, data_dir, config):
 
         self.ui_callback = ui_callback
         self.network_callback = network_callback
@@ -132,8 +131,10 @@ class NetworkEventProcessor:
         self.users = {}
 
         self.queue = queue.Queue(0)
+        self.statistics = Statistics(self.config, self.ui_callback)
         self.shares = Shares(self, self.config, self.queue, self.ui_callback)
-        self.pluginhandler = PluginHandler(self.ui_callback, plugins, self.config)
+        self.pluginhandler = None  # Initialized when the GUI is ready
+        self.now_playing = NowPlaying(self.config)
 
         script_dir = os.path.dirname(__file__)
         file_path = os.path.join(script_dir, "geoip/ipcountrydb.bin")
@@ -149,13 +150,6 @@ class NetworkEventProcessor:
         self.upnp_timer = None
         _thread.start_new_thread(self.add_upnp_portmapping, ())
 
-        uselimit = self.config.sections["transfers"]["uselimit"]
-        uploadlimit = self.config.sections["transfers"]["uploadlimit"]
-        limitby = self.config.sections["transfers"]["limitby"]
-
-        self.queue.put(slskmessages.SetUploadLimit(uselimit, uploadlimit, limitby))
-        self.queue.put(slskmessages.SetDownloadLimit(self.config.sections["transfers"]["downloadlimit"]))
-
         self.active_server_conn = None
         self.waitport = None
         self.chatrooms = None
@@ -823,8 +817,7 @@ class NetworkEventProcessor:
             self.shares.set_connected(False)
 
             if self.transfers is not None:
-                self.transfers.abort_transfers(send_fail_message=False)
-                self.transfers.save_downloads()
+                self.transfers.disconnect()
 
             self.privatechat = self.chatrooms = self.userinfo = self.userbrowse = self.search = self.transfers = self.userlist = None
             self.ui_callback.conn_close(conn, addr)
@@ -1051,9 +1044,12 @@ class NetworkEventProcessor:
             self.queue.put(slskmessages.SetStatus((not self.ui_callback.away) + 1))
 
             for thing in self.config.sections["interests"]["likes"]:
-                self.queue.put(slskmessages.AddThingILike(thing))
+                if thing and isinstance(thing, str):
+                    self.queue.put(slskmessages.AddThingILike(thing))
+
             for thing in self.config.sections["interests"]["dislikes"]:
-                self.queue.put(slskmessages.AddThingIHate(thing))
+                if thing and isinstance(thing, str):
+                    self.queue.put(slskmessages.AddThingIHate(thing))
 
             self.queue.put(slskmessages.HaveNoParent(1))
 
@@ -1065,6 +1061,12 @@ class NetworkEventProcessor:
             self.queue.put(slskmessages.NotifyPrivileges(1, self.config.sections["server"]["login"]))
             self.privatechat.login()
             self.queue.put(slskmessages.CheckPrivileges())
+
+            """ Request a complete room list. A limited room list not including blacklisted rooms and
+            rooms with few users is automatically sent when logging in, but subsequent room list
+            requests contain all rooms. """
+            self.queue.put(slskmessages.RoomList())
+
             self.queue.put(slskmessages.PrivateRoomToggle(self.config.sections["server"]["private_chatrooms"]))
         else:
             self.manualdisconnect = True
@@ -1148,14 +1150,14 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.user_joined_room(msg)
+            self.chatrooms.user_joined_room(msg)
 
     def public_room_message(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.public_room_message(msg, msg.msg)
+            self.chatrooms.public_room_message(msg, msg.msg)
             self.pluginhandler.public_room_message_notification(msg.room, msg.user, msg.msg)
 
     def join_room(self, msg):
@@ -1163,98 +1165,98 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.join_room(msg)
+            self.chatrooms.join_room(msg)
 
     def private_room_users(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_users(msg)
+            self.chatrooms.private_room_users(msg)
 
     def private_room_owned(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_owned(msg)
+            self.chatrooms.private_room_owned(msg)
 
     def private_room_add_user(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_add_user(msg)
+            self.chatrooms.private_room_add_user(msg)
 
     def private_room_remove_user(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_remove_user(msg)
+            self.chatrooms.private_room_remove_user(msg)
 
     def private_room_operator_added(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_operator_added(msg)
+            self.chatrooms.private_room_operator_added(msg)
 
     def private_room_operator_removed(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_operator_removed(msg)
+            self.chatrooms.private_room_operator_removed(msg)
 
     def private_room_add_operator(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_add_operator(msg)
+            self.chatrooms.private_room_add_operator(msg)
 
     def private_room_remove_operator(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_remove_operator(msg)
+            self.chatrooms.private_room_remove_operator(msg)
 
     def private_room_added(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_added(msg)
+            self.chatrooms.private_room_added(msg)
 
     def private_room_removed(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_removed(msg)
+            self.chatrooms.private_room_removed(msg)
 
     def private_room_disown(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.private_room_disown(msg)
+            self.chatrooms.private_room_disown(msg)
 
     def private_room_toggle(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.toggle_private_rooms(msg.enabled)
+            self.chatrooms.toggle_private_rooms(msg.enabled)
 
     def leave_room(self, msg):
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.leave_room(msg)
+            self.chatrooms.leave_room(msg)
 
     def private_message_queue_add(self, msg, text):
 
@@ -1314,9 +1316,10 @@ class NetworkEventProcessor:
 
         if self.chatrooms is not None:
             event = self.pluginhandler.incoming_public_chat_event(msg.room, msg.user, msg.msg)
+
             if event is not None:
                 (r, n, msg.msg) = event
-                self.chatrooms.roomsctrl.say_chat_room(msg, msg.msg)
+                self.chatrooms.say_chat_room(msg, msg.msg)
                 self.pluginhandler.incoming_public_chat_notification(msg.room, msg.user, msg.msg)
 
     def add_user(self, msg):
@@ -1448,7 +1451,7 @@ class NetworkEventProcessor:
             self.userbrowse.get_user_status(msg)
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.get_user_status(msg)
+            self.chatrooms.get_user_status(msg)
 
     def user_interests(self, msg):
 
@@ -1468,7 +1471,7 @@ class NetworkEventProcessor:
             self.interests.get_user_stats(msg)
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.get_user_stats(msg)
+            self.chatrooms.get_user_stats(msg)
 
         if self.userinfo is not None:
             self.userinfo.get_user_stats(msg)
@@ -1490,7 +1493,7 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.user_left_room(msg)
+            self.chatrooms.user_left_room(msg)
 
     def relogged(self, msg):
         log.add(_("Someone else is logging in with the same nickname, server is going to disconnect us"))
@@ -1519,13 +1522,10 @@ class NetworkEventProcessor:
         if self.config.sections["transfers"]["friendsonly"]:
             return 0, "Sorry, friends only"
 
-        if not self.config.sections["transfers"]["geoblock"]:
+        if ip is None or not self.config.sections["transfers"]["geoblock"]:
             return 1, ""
 
-        cc = "-"
-
-        if ip is not None:
-            cc = self.geoip.get_all(ip).country_short
+        cc = self.geoip.get_all(ip).country_short
 
         if cc == "-":
             if self.config.sections["transfers"]["geopanic"]:
@@ -1901,9 +1901,9 @@ class NetworkEventProcessor:
             return
 
         if checkuser == 1:
-            shares = self.config.sections["transfers"]["sharedfilesstreams"]
+            shares = self.shares.share_dbs["streams"]
         elif checkuser == 2:
-            shares = self.config.sections["transfers"]["bsharedfilesstreams"]
+            shares = self.shares.share_dbs["streams"]
         else:
             self.queue.put(slskmessages.TransferResponse(conn, 0, reason=reason, req=0))
             shares = {}
@@ -1915,7 +1915,7 @@ class NetworkEventProcessor:
                 self.queue.put(slskmessages.FolderContentsResponse(conn, msg.dir, shares[msg.dir.rstrip('\\')]))
             else:
                 if checkuser == 2:
-                    shares = self.config.sections["transfers"]["sharedfilesstreams"]
+                    shares = self.shares.share_dbs["streams"]
                     if msg.dir in shares:
                         self.queue.put(slskmessages.FolderContentsResponse(conn, msg.dir, shares[msg.dir]))
                     elif msg.dir.rstrip("\\") in shares:
@@ -1956,7 +1956,7 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.set_room_list(msg)
+            self.chatrooms.set_room_list(msg)
             self.set_status("")
 
     def global_user_list(self, msg):
@@ -2122,7 +2122,7 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.ticker_set(msg)
+            self.chatrooms.ticker_set(msg)
 
     def room_ticker_add(self, msg):
         """ Server code: 114 """
@@ -2130,7 +2130,7 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.ticker_add(msg)
+            self.chatrooms.ticker_add(msg)
 
     def room_ticker_remove(self, msg):
         """ Server code: 115 """
@@ -2138,7 +2138,7 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.chatrooms is not None:
-            self.chatrooms.roomsctrl.ticker_remove(msg)
+            self.chatrooms.ticker_remove(msg)
 
     def update_debug_log_options(self):
         """ Gives the logger updated logging settings """
