@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2013 eL_vErDe <gandalf@le-vert.net>
@@ -30,13 +30,11 @@ the transfer manager.
 
 import os
 import os.path
-import pickle
 import re
 import stat
 import threading
 import time
 
-from gettext import gettext as _
 from hashlib import md5
 from time import sleep
 
@@ -47,7 +45,6 @@ from pynicotine.utils import execute_command
 from pynicotine.utils import clean_file
 from pynicotine.utils import clean_path
 from pynicotine.utils import get_result_bitrate_length
-from pynicotine.utils import RestrictedUnpickler
 from pynicotine.utils import write_log
 
 
@@ -59,14 +56,14 @@ class Transfer(object):
                 "offset", "currentbytes", "lastbytes", "speed", "timeelapsed", \
                 "timeleft", "timequeued", "transfertimer", "requestconn", \
                 "modifier", "place", "bitrate", "length", "iter", "_status", \
-                "laststatuschange"
+                "laststatuschange", "legacy_attempt"
 
     def __init__(
         self, conn=None, user=None, realfilename=None, filename=None,
         path=None, status=None, req=None, size=None, file=None, starttime=None,
         offset=None, currentbytes=None, speed=None, timeelapsed=None,
         timeleft=None, timequeued=None, transfertimer=None, requestconn=None,
-        modifier=None, place=0, bitrate=None, length=None, iter=None
+        modifier=None, place=0, bitrate=None, length=None, iter=None, legacy_attempt=False
     ):
         self.user = user
         self.realfilename = realfilename  # Sent as is to the user announcing what file we're sending
@@ -92,6 +89,7 @@ class Transfer(object):
         self.bitrate = bitrate
         self.length = length
         self.iter = iter
+        self.legacy_attempt = legacy_attempt
         self.setstatus(status)
 
     def setstatus(self, status):
@@ -132,30 +130,30 @@ class Transfers:
         self.privilegedusers = set()
         userstatus = set()
 
+        self.update_limits()
+
         for i in self.load_download_queue():
             size = currentbytes = bitrate = length = None
 
-            if len(i) >= 6:
-                try:
-                    size = int(i[4])
-                except Exception:
-                    pass
+            try:
+                size = int(i[4])
+            except Exception:
+                pass
 
-                try:
-                    currentbytes = int(i[5])
-                except Exception:
-                    pass
+            try:
+                currentbytes = int(i[5])
+            except Exception:
+                pass
 
-            if len(i) >= 8:
-                try:
-                    bitrate = i[6]
-                except Exception:
-                    pass
+            try:
+                bitrate = i[6]
+            except Exception:
+                pass
 
-                try:
-                    length = i[7]
-                except Exception:
-                    pass
+            try:
+                length = i[7]
+            except Exception:
+                pass
 
             if len(i) >= 4 and i[3] in ("Aborted", "Paused"):
                 status = "Paused"
@@ -193,20 +191,68 @@ class Transfers:
         # Check for failed downloads if option is enabled (1 min delay)
         self.start_check_download_queue_timer()
 
+    def get_download_queue_file_name(self):
+
+        data_dir = self.eventprocessor.config.data_dir
+        downloads_file_json = os.path.join(data_dir, 'downloads.json')
+        downloads_file_1_4_2 = os.path.join(data_dir, 'config.transfers.pickle')
+        downloads_file_1_4_1 = os.path.join(data_dir, 'transfers.pickle')
+
+        if os.path.exists(downloads_file_json):
+            # New file format
+            return downloads_file_json
+
+        elif os.path.exists(downloads_file_1_4_2):
+            # Nicotine+ 1.4.2+
+            return downloads_file_1_4_2
+
+        elif os.path.exists(downloads_file_1_4_1):
+            # Nicotine <=1.4.1
+            return downloads_file_1_4_1
+
+        return None
+
+    def load_current_queue_format(self, downloads_file):
+        """ Loads a download queue file in json format """
+
+        download_queue = []
+
+        try:
+            with open(downloads_file, encoding="utf-8") as handle:
+                import json
+                download_queue = json.load(handle)
+
+        except Exception as inst:
+            log.add(_("Something went wrong while reading your transfer list: %(error)s"), {'error': str(inst)})
+
+        return download_queue
+
+    def load_legacy_queue_format(self, downloads_file):
+        """ Loads a download queue file in pickle format (legacy) """
+
+        download_queue = []
+
+        try:
+            with open(downloads_file, "rb") as handle:
+                from pynicotine.utils import RestrictedUnpickler
+                download_queue = RestrictedUnpickler(handle, encoding='utf-8').load()
+
+        except Exception as inst:
+            log.add(_("Something went wrong while reading your transfer list: %(error)s"), {'error': str(inst)})
+
+        return download_queue
+
     def load_download_queue(self):
 
-        if os.path.exists(os.path.join(self.eventprocessor.config.data_dir, 'transfers.pickle')):
-            try:
-                with open(os.path.join(self.eventprocessor.config.data_dir, 'transfers.pickle'), 'rb') as handle:
-                    return RestrictedUnpickler(handle, encoding='utf-8').load()
+        downloads_file = self.get_download_queue_file_name()
 
-            except IOError as inst:
-                log.add_warning(_("Something went wrong while opening your transfer list: %(error)s"), {'error': str(inst)})
+        if not downloads_file:
+            return []
 
-            except Exception as inst:
-                log.add_warning(_("Something went wrong while reading your transfer list: %(error)s"), {'error': str(inst)})
+        if not downloads_file.endswith("downloads.json"):
+            return self.load_legacy_queue_format(downloads_file)
 
-        return []
+        return self.load_current_queue_format(downloads_file)
 
     def set_transfer_views(self, downloads, uploads):
         self.downloadsview = downloads
@@ -226,6 +272,16 @@ class Transfers:
             self.privcount += self.usersqueued[user]
             del self.usersqueued[user]
 
+    def update_limits(self):
+        """ Sends the updated speed limits to the networking thread """
+
+        uselimit = self.eventprocessor.config.sections["transfers"]["uselimit"]
+        uploadlimit = self.eventprocessor.config.sections["transfers"]["uploadlimit"]
+        limitby = self.eventprocessor.config.sections["transfers"]["limitby"]
+
+        self.queue.put(slskmessages.SetUploadLimit(uselimit, uploadlimit, limitby))
+        self.queue.put(slskmessages.SetDownloadLimit(self.eventprocessor.config.sections["transfers"]["downloadlimit"]))
+
     def get_user_status(self, msg):
         """ We get a status of a user and if he's online, we request a file from him """
 
@@ -239,14 +295,6 @@ class Transfers:
                         i.status = "User logged off"
                         self.downloadsview.update(i)
 
-        for i in self.uploads[:]:
-            if msg.user == i.user and i.status != "Finished":
-                if msg.status == 0:
-                    if i.transfertimer is not None:
-                        i.transfertimer.cancel()
-                    self.uploads.remove(i)
-                    self.uploadsview.remove_specific(i, True)
-
         if msg.status == 0:
             self.check_upload_queue()
 
@@ -586,7 +634,7 @@ class Transfers:
             if self.can_upload(user):
                 path = ""
                 if self.eventprocessor.config.sections["transfers"]["uploadsinsubdirs"]:
-                    parentdir = msg.file.split("\\")[-2]
+                    parentdir = msg.file.replace('/', '\\').split('\\')[-2]
                     path = self.eventprocessor.config.sections["transfers"]["uploaddir"] + os.sep + user + os.sep + parentdir
 
                 transfer = Transfer(
@@ -759,7 +807,7 @@ class Transfers:
         Decode the incorrect parts as utf-8, if necessary, otherwise Nicotine+ thinks the file
         isn't shared. """
 
-        filename_parts = msg.file.split('\\')
+        filename_parts = msg.file.replace('/', '\\').split('\\')
 
         for i, part in enumerate(filename_parts):
             try:
@@ -887,11 +935,15 @@ class Transfers:
 
     def queue_failed(self, msg):
 
+        user = None
         for i in self.peerconns:
             if i.conn is msg.conn.conn:
                 user = i.username
                 break
 
+        if user is None:
+            return
+
         for i in self.downloads:
             if i.user == user and i.filename == msg.file and i.status not in ["Aborted", "Paused"]:
                 if i.status in self.TRANSFER:
@@ -904,8 +956,6 @@ class Transfers:
 
     def file_is_shared(self, user, virtualfilename, realfilename):
 
-        realfilename = realfilename.replace("\\", os.sep)
-
         log.add_transfer("Checking if file %(virtual_name)s with real path %(path)s is shared", {
             "virtual_name": virtualfilename,
             "path": realfilename
@@ -922,13 +972,13 @@ class Transfers:
 
         if self.eventprocessor.config.sections["transfers"]["enablebuddyshares"]:
             if user in (i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]):
-                bshared = self.eventprocessor.config.sections["transfers"]["bsharedfiles"]
+                bshared = self.eventprocessor.shares.share_dbs["buddyfiles"]
 
                 for i in bshared.get(str(folder), ''):
                     if file == i[0]:
                         return True
 
-        shared = self.eventprocessor.config.sections["transfers"]["sharedfiles"]
+        shared = self.eventprocessor.shares.share_dbs["files"]
 
         for i in shared.get(str(folder), ''):
             if file == i[0]:
@@ -996,7 +1046,7 @@ class Transfers:
     def get_file_size(self, filename):
 
         try:
-            size = os.path.getsize(filename.replace("\\", os.sep))
+            size = os.path.getsize(filename)
         except Exception:
             # file doesn't exist (remote files are always this)
             size = 0
@@ -1021,6 +1071,25 @@ class Transfers:
                 if i.req != msg.req:
                     continue
 
+                if msg.reason in ("File not shared.", "File not shared", "Remote file error") and \
+                        not i.legacy_attempt:
+                    """ The peer is possibly using an old client that doesn't support Unicode
+                    (Soulseek NS). Attempt to request file name encoded as latin-1 once. """
+
+                    i.req = new_id()
+                    realpath = self.eventprocessor.shares.virtual2real(i.filename)
+                    request = slskmessages.TransferRequest(None, 0, i.req, i.filename, self.get_file_size(realpath), realpath, legacy_client=True)
+                    self.eventprocessor.send_message_to_peer(i.user, request)
+                    i.legacy_attempt = True
+
+                    log.add_transfer("Peer responded with reason '%(reason)s' for download request %(request)s for file %(filename)s. "
+                                     "Attempting to request file as latin-1.", {
+                                         "reason": msg.reason,
+                                         "request": msg.req,
+                                         "filename": i.filename
+                                     })
+                    break
+
                 i.status = msg.reason
                 i.req = None
                 self.downloadsview.update(i)
@@ -1031,7 +1100,9 @@ class Transfers:
                         if i.user not in self.eventprocessor.watchedusers:
                             self.queue.put(slskmessages.AddUser(i.user))
 
-                    self.queue.put(slskmessages.PlaceInQueueRequest(msg.conn.conn, i.filename))
+                    self.queue.put(
+                        slskmessages.PlaceInQueueRequest(msg.conn.conn, i.filename, i.legacy_attempt)
+                    )
 
                 self.check_upload_queue()
                 break
@@ -1195,7 +1266,7 @@ class Transfers:
 
             else:
                 # also check for a windows-style incomplete transfer
-                basename = clean_file(i.filename.split('\\')[-1])
+                basename = clean_file(i.filename.replace('/', '\\').split('\\')[-1])
                 winfname = os.path.join(incompletedir, "INCOMPLETE~" + basename)
                 pyfname = os.path.join(incompletedir, "INCOMPLETE" + basename)
 
@@ -1243,8 +1314,11 @@ class Transfers:
                     i.offset = size
                     i.starttime = time.time()
 
+                    self.eventprocessor.statistics.append_stat_value("started_downloads", 1)
+
                     if i.size > size:
                         i.status = "Transferring"
+                        i.legacy_attempt = False
                         self.queue.put(slskmessages.DownloadFile(i.conn, size, f, i.size))
                         log.add_transfer("Download started: %s", f.name)
 
@@ -1259,7 +1333,7 @@ class Transfers:
                 self.downloadsview.update(i)
 
         else:
-            log.add_warning(_("Download error formally known as 'Unknown file request': %(req)s (%(user)s: %(file)s)"), {
+            log.add_warning("Download error formally known as 'Unknown file request': %(req)s (%(user)s: %(file)s)", {
                 'req': str(vars(msg)),
                 'user': i.user,
                 'file': i.filename
@@ -1284,15 +1358,20 @@ class Transfers:
 
             try:
                 # Open File
-                filename = i.realfilename.replace("\\", os.sep)
-
-                f = open(filename, "rb")
+                f = open(i.realfilename, "rb")
                 self.queue.put(slskmessages.UploadFile(i.conn, file=f, size=i.size))
                 i.status = "Initializing transfer"
                 i.file = f
 
-                self.log_transfer(_("Upload started: user %(user)s, file %(file)s") % {
+                self.eventprocessor.statistics.append_stat_value("started_uploads", 1)
+
+                ip_address = None
+                if i.conn is not None:
+                    ip_address = i.conn.getpeername()
+
+                self.log_transfer(_("Upload started: user %(user)s, IP address %(ip)s, file %(file)s") % {
                     'user': i.user,
+                    'ip': ip_address,
                     'file': i.filename
                 })
             except IOError as strerror:
@@ -1313,7 +1392,7 @@ class Transfers:
                 self.upload_finished(i, file=f)
 
         else:
-            log.add_warning(_("Upload error formally known as 'Unknown file request': %(req)s (%(user)s: %(file)s)"), {
+            log.add_warning("Upload error formally known as 'Unknown file request': %(req)s (%(user)s: %(file)s)", {
                 'req': str(vars(msg)),
                 'user': i.user,
                 'file': i.filename
@@ -1353,11 +1432,18 @@ class Transfers:
                 if curtime > i.starttime and \
                         i.currentbytes > i.lastbytes:
 
+                    bytesdifference = (i.currentbytes - i.lastbytes)
+                    self.eventprocessor.statistics.append_stat_value("downloaded_size", bytesdifference)
+
                     try:
-                        i.speed = max(0, (i.currentbytes - i.lastbytes) / (curtime - i.lasttime))
+                        i.speed = max(0, bytesdifference / (curtime - i.lasttime))
                     except ZeroDivisionError:
-                        i.speed = 0
+                        i.speed = None
+
                     if i.speed <= 0.0:
+                        i.speed = None
+
+                    if i.speed is None:
                         i.timeleft = "∞"
                     else:
                         i.timeleft = self.get_time((i.size - i.currentbytes) / i.speed)
@@ -1391,7 +1477,7 @@ class Transfers:
         file.close()
         i.file = None
 
-        basename = clean_file(i.filename.split('\\')[-1])
+        basename = clean_file(i.filename.replace('/', '\\').split('\\')[-1])
         config = self.eventprocessor.config.sections
         downloaddir = config["transfers"]["downloaddir"]
 
@@ -1419,7 +1505,7 @@ class Transfers:
 
         i.status = "Finished"
         i.currentbytes = i.size
-        i.speed = 0
+        i.speed = None
         i.timeleft = ""
 
         log.add_transfer(
@@ -1442,6 +1528,8 @@ class Transfers:
         self.add_to_shared(newname)
         self.eventprocessor.shares.send_num_shared_folders_files()
 
+        self.eventprocessor.statistics.append_stat_value("completed_downloads", 1)
+
         if self.notifications and config["notifications"]["notification_popup_file"]:
             self.notifications.new_notification(
                 _("%(file)s downloaded from %(user)s") % {
@@ -1507,7 +1595,7 @@ class Transfers:
                 i.starttime = curtime
                 i.offset = msg.offset
 
-            lastspeed = 0
+            lastspeed = None
             if i.speed is not None:
                 lastspeed = i.speed
 
@@ -1518,16 +1606,20 @@ class Transfers:
             if curtime > i.starttime and \
                     i.currentbytes > i.lastbytes:
 
+                bytesdifference = (i.currentbytes - i.lastbytes)
+                self.eventprocessor.statistics.append_stat_value("uploaded_size", bytesdifference)
+
                 try:
-                    i.speed = max(0, (i.currentbytes - i.lastbytes) / (curtime - i.lasttime))
+                    i.speed = max(0, bytesdifference / (curtime - i.lasttime))
                 except ZeroDivisionError:
                     i.speed = lastspeed  # too fast!
 
-                if i.speed <= 0.0 and (i.currentbytes != i.size or lastspeed == 0):
+                if i.speed <= 0.0:
+                    i.speed = None
+
+                if i.speed is None:
                     i.timeleft = "∞"
                 else:
-                    if (i.currentbytes == i.size) and i.speed == 0:
-                        i.speed = lastspeed
                     i.timeleft = self.get_time((i.size - i.currentbytes) / i.speed)
 
                 self.check_upload_queue()
@@ -1567,9 +1659,13 @@ class Transfers:
         if file is not None:
             file.close()
 
+        ip_address = None
+        if i.conn is not None:
+            ip_address = i.conn.getpeername()
+
         i.status = "Finished"
         i.currentbytes = i.size
-        i.speed = 0
+        i.speed = None
         i.timeleft = ""
 
         for j in self.uploads:
@@ -1577,12 +1673,15 @@ class Transfers:
                 j.timequeued = i.lasttime
 
         self.log_transfer(
-            _("Upload finished: %(user)s, file %(file)s") % {
+            _("Upload finished: user %(user)s, IP address %(ip)s, file %(file)s") % {
                 'user': i.user,
+                'ip': ip_address,
                 'file': i.filename
             }
         )
 
+        self.eventprocessor.statistics.append_stat_value("completed_uploads", 1)
+
         self.check_upload_queue()
         self.uploadsview.update(i)
 
@@ -1635,10 +1734,11 @@ class Transfers:
             self.eventprocessor.config.write_configuration()
 
     def start_check_download_queue_timer(self):
-        timer = threading.Timer(60.0, self.ui_callback, [[slskmessages.CheckDownloadQueue()]])
-        timer.setName("DownloadQueueTimer")
-        timer.setDaemon(True)
-        timer.start()
+
+        self.download_queue_timer = threading.Timer(60.0, self.ui_callback, [[slskmessages.CheckDownloadQueue()]])
+        self.download_queue_timer.setName("DownloadQueueTimer")
+        self.download_queue_timer.setDaemon(True)
+        self.download_queue_timer.start()
 
     # Find failed or stuck downloads and attempt to queue them.
     # Also ask for the queue position of downloads.
@@ -1653,7 +1753,10 @@ class Transfers:
                 self.get_file(transfer.user, transfer.filename, transfer.path, transfer)
 
             elif transfer.status == "Queued":
-                self.eventprocessor.send_message_to_peer(transfer.user, slskmessages.PlaceInQueueRequest(None, transfer.filename))
+                self.eventprocessor.send_message_to_peer(
+                    transfer.user,
+                    slskmessages.PlaceInQueueRequest(None, transfer.filename, transfer.legacy_attempt)
+                )
 
         self.start_check_download_queue_timer()
 
@@ -1913,6 +2016,7 @@ class Transfers:
             i.file.close()
 
         i.conn = None
+        i.legacy_attempt = False
 
         if i.status != "Finished":
             if i.user in self.users and self.users[i.user].status == 0:
@@ -2020,28 +2124,14 @@ class Transfers:
             for directory in file_list[i]:
 
                 if os.path.commonprefix([i, directory]) == directory:
-                    priorityfiles = []
-                    normalfiles = []
-
-                    if self.eventprocessor.config.sections["transfers"]["prioritize"]:
-                        for file in file_list[i][directory]:
-                            parts = file[1].rsplit('.', 1)
-
-                            if len(parts) == 2 and parts[1] in ['sfv', 'md5', 'nfo']:
-                                priorityfiles.append(file)
-                            else:
-                                normalfiles.append(file)
-                    else:
-                        normalfiles = file_list[i][directory][:]
+                    files = file_list[i][directory][:]
 
                     if self.eventprocessor.config.sections["transfers"]["reverseorder"]:
-                        deco = [(x[1], x) for x in normalfiles]
-                        deco.sort(reverse=True)
-                        normalfiles = [x for junk, x in deco]
+                        files.sort(key=lambda x: x[1], reverse=True)
 
-                    for file in priorityfiles + normalfiles:
+                    for file in files:
                         size = file[2]
-                        h_bitrate, bitrate, h_length = get_result_bitrate_length(size, file[4])
+                        h_bitrate, bitrate, h_length, length = get_result_bitrate_length(size, file[4])
 
                         if directory[-1] == '\\':
                             self.get_file(
@@ -2110,8 +2200,9 @@ class Transfers:
 
     def abort_transfer(self, transfer, remove=False, reason="Aborted", send_fail_message=True):
 
+        transfer.legacy_attempt = False
         transfer.req = None
-        transfer.speed = 0
+        transfer.speed = None
         transfer.timeleft = ""
 
         if send_fail_message and transfer in self.uploads:
@@ -2167,14 +2258,54 @@ class Transfers:
         """ Save list of files to be downloaded """
 
         self.eventprocessor.config.create_data_folder()
-        downloads_file = os.path.join(self.eventprocessor.config.data_dir, 'transfers.pickle')
+        downloads_file = os.path.join(self.eventprocessor.config.data_dir, 'downloads.json')
 
         try:
-            with open(downloads_file, 'wb') as handle:
-                pickle.dump(self.get_downloads(), handle, protocol=pickle.HIGHEST_PROTOCOL)
-
-        except IOError as inst:
-            log.add_warning(_("Something went wrong while opening your transfer list: %(error)s"), {'error': str(inst)})
+            with open(downloads_file, "w", encoding="utf-8") as handle:
+                import json
+                json.dump(self.get_downloads(), handle, ensure_ascii=False)
 
         except Exception as inst:
-            log.add_warning(_("Something went wrong while writing your transfer list: %(error)s"), {'error': str(inst)})
+            log.add(_("Something went wrong while writing your transfer list: %(error)s"), {'error': str(inst)})
+
+    def disconnect(self):
+
+        if self.download_queue_timer is not None:
+            self.download_queue_timer.cancel()
+
+        self.abort_transfers(send_fail_message=False)
+        self.save_downloads()
+
+
+class Statistics:
+
+    def __init__(self, config, ui_callback=None):
+
+        self.config = config
+        self.ui_callback = ui_callback
+
+        self.init_stats()
+
+    def init_stats(self):
+        for stat_id in self.config.defaults["statistics"]:
+            self.__dict__[stat_id] = 0
+
+    def append_stat_value(self, stat_id, stat_value):
+
+        self.__dict__[stat_id] += stat_value
+        self.config.sections["statistics"][stat_id] += stat_value
+        self.update_ui(stat_id)
+
+    def update_ui(self, stat_id):
+
+        if self.ui_callback:
+            stat_value = self.__dict__[stat_id]
+            self.ui_callback.update_stat_value(stat_id, stat_value)
+
+    def reset_stats(self):
+
+        for stat_id in self.config.defaults["statistics"]:
+            self.__dict__[stat_id] = 0
+            self.config.sections["statistics"][stat_id] = 0
+
+            self.update_ui(stat_id)
