@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2020 Lene Preuss <lene.preuss@gmail.com>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2007 Daelstorm <daelstorm@gmail.com>
@@ -28,15 +28,11 @@ This module contains utility functions.
 import gettext
 import locale
 import os
+import pickle
 import sys
 import time
 
-from codecs import encode, decode
-from gettext import gettext as _
-from subprocess import PIPE
-from subprocess import Popen
-
-version = "2.1.2"
+version = "3.0.0"
 
 win32 = sys.platform.startswith("win")
 
@@ -82,17 +78,21 @@ def clean_path(path, absolute=False):
 
 def get_latest_version():
 
-    import http.client
     import json
 
-    conn = http.client.HTTPSConnection("api.github.com")
-    conn.request("GET", "/repos/Nicotine-Plus/nicotine-plus/releases/latest", headers={"User-Agent": "Nicotine+"})
-    resp = conn.getresponse()
-    data = json.loads(resp.read().decode("utf-8"))
+    response = http_request(
+        "https", "pypi.org", "/pypi/nicotine-plus/json",
+        headers={"User-Agent": "Nicotine+"}
+    )
+    data = json.loads(response)
 
-    hlatest = data['tag_name']
+    hlatest = data['info']['version']
     latest = int(make_version(hlatest))
-    date = data['created_at']
+
+    try:
+        date = data['releases'][hlatest][0]['upload_time']
+    except Exception:
+        date = None
 
     return hlatest, latest, date
 
@@ -154,6 +154,7 @@ def get_result_bitrate_length(filesize, attributes):
     h_length = ""
 
     bitrate = 0
+    length = 0
 
     # If there are 3 entries in the attribute list
     if len(attributes) == 3:
@@ -171,6 +172,7 @@ def get_result_bitrate_length(filesize, attributes):
             bitrate = first
             h_bitrate = str(bitrate) + h_bitrate
 
+            length = second
             h_length = '%i:%02i' % (second / 60, second % 60)
 
         # Sometimes the vbr indicator is in second position
@@ -182,6 +184,7 @@ def get_result_bitrate_length(filesize, attributes):
             bitrate = first
             h_bitrate = str(bitrate) + h_bitrate
 
+            length = third
             h_length = '%i:%02i' % (third / 60, third % 60)
 
         # Lossless audio, length is in first position
@@ -192,6 +195,7 @@ def get_result_bitrate_length(filesize, attributes):
             bitrate = (second * third * 2) / 1000
             h_bitrate = str(bitrate)
 
+            length = first
             h_length = '%i:%02i' % (first / 60, first % 60)
 
         else:
@@ -222,8 +226,11 @@ def get_result_bitrate_length(filesize, attributes):
                 bitrate = first
                 h_bitrate = str(bitrate) + h_bitrate
 
-                # Dividing the file size by the bitrate in Bytes should give us a good enough approximation
-                length = filesize / (bitrate / 8 * 1000)
+                if bitrate <= 0:
+                    length = 0
+                else:
+                    # Dividing the file size by the bitrate in Bytes should give us a good enough approximation
+                    length = filesize / (bitrate / 8 * 1000)
 
                 h_length = '%i:%02i' % (length / 60, length % 60)
 
@@ -233,9 +240,10 @@ def get_result_bitrate_length(filesize, attributes):
             bitrate = first
             h_bitrate = str(bitrate) + h_bitrate
 
+            length = second
             h_length = '%i:%02i' % (second / 60, second % 60)
 
-    return h_bitrate, bitrate, h_length
+    return h_bitrate, bitrate, h_length, length
 
 
 def apply_translation():
@@ -255,51 +263,19 @@ def apply_translation():
     Note: To the best of my knowledge when we are in a python venv
     falling back to the system path does not work."""
 
+    # Locales handling: We let the system handle the locales
+    try:
+        locale.setlocale(locale.LC_ALL, '')
+
+    except Exception as e:
+        print("Error while attempting to set locale: %s" % e)
+
     # package name for gettext
     package = 'nicotine'
 
     # Local path where to find translation (mo) files
     local_mo_path = 'mo'
 
-    # Python 2.7.X is build via Visual Studio 2008 on Windows:
-    # https://stackoverflow.com/questions/32037573/load-gtk-glade-translations-in-windows-using-python-pygobject
-    # https://docs.python.org/devguide/setup.html#windows
-    # The locales table for VS2008 can be found here:
-    # https://msdn.microsoft.com/en-us/library/cdax410z(v=vs.90).aspx
-    # https://msdn.microsoft.com/en-us/library/39cwe7zf(v=vs.90).aspx
-    def _build_localename_win(localetuple):
-        """ Builds a locale code from the given tuple (language code, encoding).
-            No aliasing or normalizing takes place."""
-
-        language, encoding = localetuple
-
-        if language is None:
-            language = 'C'
-
-        if encoding is None:
-            return language
-        else:
-            return language + '.' + encoding
-
-    # Locales handling
-    if win32:
-
-        # On windows python can get a normalize tuple (language code, encoding)
-        locale_win = locale.getdefaultlocale()
-
-        # Build a locale name compatible with gettext
-        locale_win_gettext = _build_localename_win(locale_win)
-
-        # Fix environnement variables
-        os.environ['LC_ALL'] = locale_win_gettext
-
-    else:
-        # Unix locales handling: We let the system handle the locales
-        try:
-            locale.setlocale(locale.LC_ALL, '')
-        except Exception as e:
-            print("Error while attempting to set locale: %s" % e)
-
     # Gettext handling
     if gettext.find(package, localedir=local_mo_path) is None:
 
@@ -311,22 +287,8 @@ def apply_translation():
     else:
 
         # Locales are in the current dir: install them
-        if win32:
-
-            # On windows we use libintl-8.dll: the core DLL of GNU gettext-runtime on Windows
-            import ctypes
-
-            libintl = ctypes.cdll.LoadLibrary("libintl-8.dll")
-
-            libintl.bindtextdomain(package, local_mo_path)
-            libintl.bind_textdomain_codeset(package, "UTF-8")
-
-        else:
-            locale.bindtextdomain(package, local_mo_path)
-            gettext.bindtextdomain(package, local_mo_path)
-
-        tr = gettext.translation(package, localedir=local_mo_path)
-        tr.install()
+        gettext.bindtextdomain(package, local_mo_path)
+        gettext.install(package, local_mo_path)
 
     gettext.textdomain(package)
 
@@ -334,10 +296,14 @@ def apply_translation():
 def unescape(string):
     """Removes quotes from the beginning and end of strings, and unescapes it."""
 
-    string = decode(encode(string, 'latin-1', 'backslashreplace'), 'unicode-escape')
+    string = string.encode('latin-1', 'backslashreplace').decode('unicode-escape')
+
+    try:
+        if (string[0] == string[-1]) and string.startswith(("'", '"')):
+            return string[1:-1]
+    except IndexError:
+        pass
 
-    if (string[0] == string[-1]) and string.startswith(("'", '"')):
-        return string[1:-1]
     return string
 
 
@@ -366,6 +332,9 @@ def execute_command(command, replacement=None, background=True, returnoutput=Fal
     * mplayer $
     * echo $ | flite -t """
 
+    from subprocess import PIPE
+    from subprocess import Popen
+
     # Example command: "C:\Program Files\WinAmp\WinAmp.exe" --xforce "--title=My Title" $ | flite -t
     if returnoutput:
         background = False
@@ -438,28 +407,66 @@ def execute_command(command, replacement=None, background=True, returnoutput=Fal
     return procs[-1].communicate()[0]
 
 
-def cmp(a, b):
-    """Replacement for cmp() which is removed in Python 3"""
-    return (a > b) - (a < b)
-
-
 def write_log(logsdir, fn, msg, timestamp_format="%Y-%m-%d %H:%M:%S"):
 
-    oldumask = os.umask(0o077)
-    if not os.path.exists(logsdir):
-        os.makedirs(logsdir)
-
     try:
+        oldumask = os.umask(0o077)
+        if not os.path.exists(logsdir):
+            os.makedirs(logsdir)
+
         with open(os.path.join(logsdir, clean_file(fn.replace(os.sep, "-")) + ".log"), 'ab', 0) as logfile:
             os.umask(oldumask)
 
             text = "%s %s\n" % (time.strftime(timestamp_format), msg)
-            logfile.write(text.encode('UTF-8', 'replace'))
+            logfile.write(text.encode('utf-8', 'replace'))
 
-    except IOError as error:
+    except Exception as error:
         print(_("Couldn't write to log file \"%s\": %s") % (fn, error))
 
 
+def http_request(url_scheme, base_url, path, request_type="GET", body="", headers={}, timeout=10, redirect_depth=0):
+
+    if redirect_depth > 15:
+        raise Exception("Redirected too many times, giving up")
+
+    import http.client
+
+    if url_scheme == "https":
+        conn = http.client.HTTPSConnection(base_url, timeout=timeout)
+    else:
+        conn = http.client.HTTPConnection(base_url, timeout=timeout)
+
+    conn.request(request_type, path, body=body, headers=headers)
+    response = conn.getresponse()
+    redirect = response.getheader('Location')
+
+    if redirect:
+        from urllib.parse import urlparse
+        parsed_url = urlparse(redirect)
+        redirect_depth += 1
+
+        return http_request(
+            parsed_url.scheme, parsed_url.netloc, parsed_url.path,
+            request_type, body, headers, timeout, redirect_depth
+        )
+
+    contents = response.read().decode("utf-8")
+    conn.close()
+
+    return contents
+
+
+class RestrictedUnpickler(pickle.Unpickler):
+    """
+    Don't allow code execution from pickles
+    """
+
+    def find_class(self, module, name):
+        # Forbid all globals
+        raise pickle.UnpicklingError("global '%s.%s' is forbidden" %
+                                     (module, name))
+
+
 """ Debugging """
 
 
